import urllib
from unittest.mock import MagicMock, patch

from fastapi.testclient import TestClient
from httpx import Response

from monarch_py.api.search import router
from monarch_py.datamodels.category_enums import MappingPredicate
from monarch_py.datamodels.model import MappingResults

client = TestClient(router)


def test_mappings(search):
    with patch.object(
        client, "get", MagicMock(return_value=Response(200, json=search, headers={"content-type": "application/json"}))
    ):
        response = client.get("/mappings?entity_id=MONDO:0000015")
    assert response.status_code == 200
    assert response.headers["content-type"] == "application/json"
    assert response.json() == search


@patch("monarch_py.implementations.solr.solr_implementation.SolrImplementation.get_mappings")
def test_mappings_params(mock_get_mappings, mappings):
    mock_get_mappings.return_value = MagicMock()
    params = {
        "entity_id": ["MONDO:0000015", "MONDO:0000016"],
        "subject_id": ["HGNC:0000001", "HGNC:0000002"],
        "predicate_id": ["skos:exactMatch"],
        "object_id": ["MONDO:0000001", "MONDO:0000002"],
        "mapping_justification": ["semapv:UnspecifiedMatching"],
        "offset": 0,
        "limit": 20,
    }
    query_string = urllib.parse.urlencode(params, doseq=True)
    mappings = MappingResults(**mappings)
    client.get(f"/mappings?{query_string}")
    mock_get_mappings.assert_called_with(
        entity_id=["MONDO:0000015", "MONDO:0000016"],
        subject_id=["HGNC:0000001", "HGNC:0000002"],
        predicate_id=[MappingPredicate.EXACT_MATCH],
        object_id=["MONDO:0000001", "MONDO:0000002"],
        mapping_justification=["semapv:UnspecifiedMatching"],
        offset=0,
        limit=20,
    )


@patch("monarch_py.implementations.solr.solr_implementation.SolrImplementation.get_mappings")
def test_empty_response(mock_get_mappings):
    empty_mapping_result = MappingResults(items=[], offset=0, limit=20, total=0)
    mock_get_mappings.return_value = empty_mapping_result

    response = client.get("/mappings?entity_id=NONEXISTENT:ID")
    assert response.status_code == 200
    assert response.json() == {"items": [], "offset": 0, "limit": 20, "total": 0}
    assert response.text == '{"limit":20,"offset":0,"total":0,"items":[]}'
    mock_get_mappings.assert_called_once()


@patch("monarch_py.implementations.solr.solr_implementation.SolrImplementation.get_mappings")
def test_tsv_format(mock_get_mappings, mappings):
    mapping_res = MappingResults(**mappings)
    mock_get_mappings.return_value = mapping_res
    with patch("monarch_py.api.search.to_tsv") as mock_to_tsv:
        expected_result = "header1\theader2\nvalue1\tvalue2\n"
        mock_to_tsv.return_value = expected_result
        response = client.get("/mappings?entity_id=ANY:0000015&format=tsv")
        assert response.status_code == 200
        assert response.headers["content-type"] == "text/tab-separated-values; charset=utf-8"
        assert response.text == expected_result
        mock_to_tsv.assert_called_with(mapping_res, print_output=False)



import pytest
from monarch_py.implementations.solr.solr_implementation import SolrImplementation

pytestmark = pytest.mark.skipif(
    condition=not SolrImplementation().solr_is_available(),
    reason="Solr is not available",
)


def test_histopheno():
    si = SolrImplementation()
    hp = si.get_histopheno("MONDO:0020121")

    total = 0
    for k in hp.items:
        total += k.count

    assert hp.items[0].id == "UPHENO:0002816"




import pytest

@pytest.fixture
def association_query_params():
    return {'category': ['biolink:DiseaseToPhenotypicFeatureAssociation'], 'subject': ['TEST:0000001'], 'subject_closure': 'TEST:0000003', 'subject_category': ['biolink:Gene'], 'subject_namespace': 'TEST', 'subject_taxon': ['NCBITaxon:1111'], 'predicate': ['biolink:causes'], 'object': ['TEST:0000002'], 'object_closure': 'TEST:0000004', 'object_category': ['biolink:Disease'], 'object_namespace': 'TEST', 'object_taxon': ['NCBITaxon:2222'], 'entity': ['TEST:0000005'], 'q': 'test:q', 'offset': 100, 'limit': 100}



import pytest

from monarch_py.implementations.solr.solr_implementation import SolrImplementation


@pytest.mark.parametrize(
    "text,expected_id",
    [
        ("Marfan syndrome", "MONDO:0007947"),
        ("Ehlers-Danlos syndrome", "MONDO:0020066"),
        ("Loeys-Dietz syndrome", "MONDO:0018954"),
        ("connective tissue disorder", "MONDO:0003900"),
    ],
)
@pytest.mark.skipif(
    condition=not SolrImplementation().solr_is_available(),
    reason="Solr is not available",
)
def test_grounding(text, expected_id):
    matching_results = SolrImplementation().ground_entity(text)
    assert matching_results
    matching_identifiers = [result.id for result in matching_results]
    assert expected_id in matching_identifiers


@pytest.mark.parametrize(
    "text,unwanted_id",
    [
        ("patients", "MGI:1332635"),
        ("reduced", "MONDO:0012143"),
        ("Bard", "MONDO:0014432"),
    ],
)
@pytest.mark.skipif(
    condition=not SolrImplementation().solr_is_available(),
    reason="Solr is not available",
)
def test_grounding_should_not_match(text, unwanted_id):
    matching_results = SolrImplementation().ground_entity(text)
    identifiers = [result.id for result in matching_results]
    assert unwanted_id not in identifiers or len(identifiers) == 0




import pytest

@pytest.fixture
def mapping_response():
    return {'responseHeader': {'QTime': 0, 'params': {'mm': '100%', 'q': '*:*', 'defType': 'edismax', 'facet_min_count': '1', 'hl': 'false', 'start': '0', 'q.op': 'AND', 'fq': 'subject_id:"MONDO\\:0020121" OR object_id:"MONDO\\:0020121"', 'facet.mincount': '1', 'rows': '20', 'facet': 'true'}}, 'response': {'num_found': 9, 'start': 0, 'docs': [{'subject_id': 'MONDO:0020121', 'subject_label': 'muscular dystrophy', 'predicate_id': 'skos:exactMatch', 'object_id': 'Orphanet:98473', 'object_label': 'Muscular dystrophy', 'mapping_justification': 'semapv:UnspecifiedMatching', 'mapping_source': ['mondo.sssom'], 'id': '303ae2ac-9e44-4908-a03a-3abf85e3eb7c', 'highlighting': None}, {'subject_id': 'MONDO:0020121', 'subject_label': 'muscular dystrophy', 'predicate_id': 'skos:exactMatch', 'object_id': 'icd11.foundation:1464662404', 'mapping_justification': 'semapv:UnspecifiedMatching', 'mapping_source': ['mondo.sssom'], 'id': '07d3ed5f-ffee-4260-9634-88baf2d0efd7', 'highlighting': None}, {'subject_id': 'MONDO:0020121', 'subject_label': 'muscular dystrophy', 'predicate_id': 'skos:exactMatch', 'object_id': 'MEDGEN:44527', 'mapping_justification': 'semapv:UnspecifiedMatching', 'mapping_source': ['mondo.sssom'], 'id': '9df8e97f-c647-4722-9d11-282c2f075db0', 'highlighting': None}, {'subject_id': 'MONDO:0020121', 'subject_label': 'muscular dystrophy', 'predicate_id': 'skos:exactMatch', 'object_id': 'DOID:9884', 'object_label': 'muscular dystrophy', 'mapping_justification': 'semapv:UnspecifiedMatching', 'mapping_source': ['mondo.sssom'], 'id': '1168db96-6f5e-4314-9ac7-bb336773a63a', 'highlighting': None}, {'subject_id': 'MONDO:0020121', 'subject_label': 'muscular dystrophy', 'predicate_id': 'skos:exactMatch', 'object_id': 'MESH:D009136', 'mapping_justification': 'semapv:UnspecifiedMatching', 'mapping_source': ['mondo.sssom'], 'id': 'e0234338-69e4-4d5c-ad84-bee6a8d63e60', 'highlighting': None}, {'subject_id': 'MONDO:0020121', 'subject_label': 'muscular dystrophy', 'predicate_id': 'skos:exactMatch', 'object_id': 'NCIT:C84910', 'object_label': 'Muscular Dystrophy', 'mapping_justification': 'semapv:UnspecifiedMatching', 'mapping_source': ['mondo.sssom'], 'id': 'd1c63d51-0f60-4655-8adb-6afe7bf25e82', 'highlighting': None}, {'subject_id': 'MONDO:0020121', 'subject_label': 'muscular dystrophy', 'predicate_id': 'skos:exactMatch', 'object_id': 'SCTID:73297009', 'mapping_justification': 'semapv:UnspecifiedMatching', 'mapping_source': ['mondo.sssom'], 'id': '20b96ad0-4d60-4ef4-b0ee-01cc333b7018', 'highlighting': None}, {'subject_id': 'MONDO:0020121', 'subject_label': 'muscular dystrophy', 'predicate_id': 'skos:exactMatch', 'object_id': 'UMLS:C0026850', 'mapping_justification': 'semapv:UnspecifiedMatching', 'mapping_source': ['mondo.sssom'], 'id': '76a45fb2-63f3-4d3f-abae-a3bddd37919f', 'highlighting': None}, {'subject_id': 'MONDO:0020121', 'subject_label': 'muscular dystrophy', 'predicate_id': 'skos:exactMatch', 'object_id': 'ICD10CM:G71.0', 'object_label': 'Muscular dystrophy', 'mapping_justification': 'semapv:UnspecifiedMatching', 'mapping_source': ['mondo.sssom'], 'id': 'afbdaf20-a6f8-49a8-b1b5-4c85478b9f19', 'highlighting': None}]}, 'facet_counts': {'facet_fields': {}, 'facet_queries': {}}, 'highlighting': {}}



from unittest.mock import MagicMock, patch

from fastapi.testclient import TestClient

from monarch_py.api.entity import router
from monarch_py.datamodels.model import Node
from monarch_py.datamodels.category_enums import AssociationCategory

client = TestClient(router)


@patch("monarch_py.implementations.solr.solr_implementation.SolrImplementation.get_entity")
def test_entity(mock_get_entity, node):
    mock_get_entity.return_value = Node(**node)
    client.get("/MONDO:0019391")
    mock_get_entity.assert_called_with("MONDO:0019391", extra=True)


@patch("monarch_py.implementations.solr.solr_implementation.SolrImplementation.get_association_table")
def test_association_table(mock_get_assoc_table):
    mock_get_assoc_table.return_value = MagicMock()
    client.get("/MONDO:0019391/biolink:DiseaseToPhenotypicFeatureAssociation")
    mock_get_assoc_table.assert_called_with(
        entity="MONDO:0019391",
        category=AssociationCategory.DISEASE_TO_PHENOTYPIC_FEATURE_ASSOCIATION,
        q=None,
        traverse_orthologs=False,
        direct=False,
        facet_fields=None,
        facet_queries=None,
        filter_queries=None,
        sort=None,
        offset=0,
        limit=20,
    )



import os
from glob import glob


def _as_module(fixture_path: str) -> str:
    return fixture_path.replace("/", ".").replace("\\", ".").replace(".py", "")


if os.getcwd().endswith("monarch-app"):
    fixtures_dir = "backend/tests/fixtures"
elif os.getcwd().endswith("backend"):
    fixtures_dir = "tests/fixtures"
else:
    raise Exception("Could not find fixtures directory")


fixtures = glob(f"{fixtures_dir}/[!_]*.py")

pytest_plugins = [_as_module(f) for f in fixtures]



import pytest

from monarch_py.implementations.solr.solr_implementation import SolrImplementation
from monarch_py.datamodels.category_enums import AssociationCategory, AssociationPredicate, EntityCategory

pytestmark = pytest.mark.skipif(
    condition=not SolrImplementation().solr_is_available(),
    reason="Solr is not available",
)


def test_associations():
    si = SolrImplementation()
    response = si.get_associations()
    assert response
    assert response.total > 100000
    assert len(response.items) == 20


def test_association_page_limit():
    si = SolrImplementation()
    response = si.get_associations(limit=100)
    assert len(response.items) == 100


def test_association_category():
    si = SolrImplementation()
    response = si.get_associations(category=[AssociationCategory.CORRELATED_GENE_TO_DISEASE_ASSOCIATION])
    assert response
    assert response.total > 6000
    assert "biolink:CorrelatedGeneToDiseaseAssociation" in response.items[0].category


def test_association_predicate():
    si = SolrImplementation()
    response = si.get_associations(predicate=[AssociationPredicate.HAS_PHENOTYPE])
    assert response
    assert response.total > 600000
    assert "biolink:has_phenotype" in response.items[0].predicate


def test_subject():
    si = SolrImplementation()
    response = si.get_associations(subject="MONDO:0007947", direct=True)
    assert response
    assert response.total > 50
    assert response.items[0].subject == "MONDO:0007947"


def test_object():
    si = SolrImplementation()
    response = si.get_associations(object="MONDO:0007947", direct=True)
    assert response
    assert response.total > 0
    assert response.items[0].object == "MONDO:0007947"


def test_object_closure():
    si = SolrImplementation()
    response = si.get_associations(object="HP:0000240")
    assert response
    assert response.total in range(200, 10000)


def test_entity():
    si = SolrImplementation()
    response = si.get_associations(entity="MONDO:0007947")
    assert response
    assert response.total > 50
    for association in response.items:
        if (
            association.subject_closure is None or len(association.subject_closure) == 0
        ) and association.disease_context_qualifier is None:
            assert "MONDO:0007947" in association.object_closure
        elif (
            association.object_closure is None or len(association.object_closure) == 0
        ) and association.disease_context_qualifier is None:
            assert "MONDO:0007947" in association.subject_closure
        elif association.disease_context_qualifier is not None:
            assert "MONDO:0007947" in association.disease_context_qualifier_closure
        else:
            assert "MONDO:0007947" in association.subject_closure or "MONDO:0007947" in association.object_closure


@pytest.mark.parametrize("q", ["eyebrow", "thick", "Thick", "Thick eyebrow", "thick eyebrow"])
def test_association_search_partial_match(q: str):
    si = SolrImplementation()
    response = si.get_associations(
        q=q, subject="MONDO:0011518", category=[AssociationCategory.DISEASE_TO_PHENOTYPIC_FEATURE_ASSOCIATION]
    )
    assert response
    assert response.total > 0
    assert "HP:0000574" in [item.object for item in response.items]


@pytest.mark.parametrize("q", ["eyebrow", "thick", "Thick", "Thick eyebrow", "thick eyebrow"])
def test_association_table_search_partial_match(q: str):
    si = SolrImplementation()
    response = si.get_association_table(
        entity="MONDO:0011518",
        category=AssociationCategory.DISEASE_TO_PHENOTYPIC_FEATURE_ASSOCIATION,
        q=q,
    )
    assert response
    assert response.total > 0
    assert "HP:0000574" in [item.object for item in response.items]


@pytest.mark.skip(reason="This endpoint is officially not supported yet")
def test_multi_entity_associations():
    si = SolrImplementation()
    response = si.get_multi_entity_associations(
        entity=["MONDO:0012933", "MONDO:0005439", "MANDO:0001138"],
        counterpart_category=[EntityCategory.GENE, EntityCategory.DISEASE],
    )
    assert response
    assert len(response) == 3
    assert response[2].name == "Entity not found"
    # assert response[0].associated_categories['biolink:Disease'].total > 0
    for c in response[0].associated_categories:
        if c.counterpart_category == "biolink:Disease":
            assert c.total > 0




import pytest

@pytest.fixture
def association_counts():
    return {'items': [{'label': 'Disease to Phenotype', 'count': 4161, 'category': 'biolink:DiseaseToPhenotypicFeatureAssociation'}, {'label': 'Causal Gene', 'count': 130, 'category': 'biolink:CausalGeneToDiseaseAssociation'}, {'label': 'Correlated Gene', 'count': 150, 'category': 'biolink:CorrelatedGeneToDiseaseAssociation'}, {'label': 'Variant to Disease', 'count': 435, 'category': 'biolink:VariantToDiseaseAssociation'}, {'label': 'Disease Model', 'count': 244, 'category': 'biolink:GenotypeToDiseaseAssociation'}, {'label': 'Medical Action', 'count': 4, 'category': 'biolink:ChemicalOrDrugOrTreatmentToDiseaseOrPhenotypicFeatureAssociation'}, {'label': 'Cases', 'count': 136, 'category': 'biolink:CaseToDiseaseAssociation'}]}



import pytest

from monarch_py.datamodels.category_enums import AssociationCategory, EntityCategory
from monarch_py.datamodels.model import AssociationDirectionEnum
from monarch_py.implementations.solr.solr_implementation import SolrImplementation

pytestmark = pytest.mark.skipif(
    condition=not SolrImplementation().solr_is_available(),
    reason="Solr is not available",
)


@pytest.mark.parametrize(
    "query",
    [
        "fgf8a",
        "MONDO:0015938",
        "Alzheimer disease 16",
        "breast cancer",
        # Failing because we don't have synonyms coming in from Phenio (!)
        #        "AD16",
        "symphalangism, C. S. Lewis type",
        "OMIM:301045",  # Confirmation that we load & search xrefs for Mondo
    ],
)
def test_search_query_has_some_results(query: str):
    si = SolrImplementation()
    response = si.search(query)
    assert response
    assert response.total > 0


def test_search_sort():
    si = SolrImplementation()
    response = si.search("marfan", sort="name desc")
    assert response
    assert response.total > 0
    assert response.items[0].name > response.items[-1].name


def test_search_highlighting():
    si = SolrImplementation()
    response = si.search("fgf8a", highlighting=True)
    assert response
    assert response.total > 0
    first_doc = response.items[0]
    assert first_doc
    assert "name" in first_doc.highlighting


def test_facet_fields():
    si = SolrImplementation()
    response = si.search("syndrome", facet_fields=["category", "in_taxon"])
    assert response
    assert response.total > 0
    assert response.facet_fields
    category = [ff for ff in response.facet_fields if ff.label == "category"][0]
    assert category


def test_single_filter_queries():
    si = SolrImplementation()
    response = si.search("syndrome", category=[EntityCategory.DISEASE])
    assert response
    assert response.total > 0
    assert response.items[0].category == "biolink:Disease"


def test_multiple_filter_queries():
    si = SolrImplementation()
    response = si.search("eye", category=[EntityCategory.DISEASE, EntityCategory.PHENOTYPIC_FEATURE])
    assert response
    assert response.total > 0
    for i, item in enumerate(response.items):
        assert item.category in ["biolink:Disease", "biolink:PhenotypicFeature"]


def test_association_facets():
    si = SolrImplementation()
    response = si.get_association_facets(facet_fields=["category"])
    assert response
    assert response.facet_fields
    category = [ff for ff in response.facet_fields if ff.label == "category"][0]
    assert category
    d2p = [fv for fv in category.facet_values if fv.label == "biolink:DiseaseToPhenotypicFeatureAssociation"][0]
    assert d2p
    assert d2p.count > 100000


def test_association_facet_query():
    si = SolrImplementation()

    response = si.get_association_facets(
        subject_closure="MONDO:0007947",
        facet_queries=[
            'object_closure:"HP:0000924"',
            'object_closure:"HP:0000707"',
            'object_closure:"HP:0000152"',
            'object_closure:"HP:0001574"',
            'object_closure:"HP:0000478"',
        ],
    )
    assert response
    assert response.facet_queries
    hp924 = [fq for fq in response.facet_queries if fq.label == 'object_closure:"HP:0000924"'][0]
    assert hp924.count > 20

    hp707 = [fq for fq in response.facet_queries if fq.label == 'object_closure:"HP:0000707"'][0]
    assert hp707.count > 5

    hp152 = [fq for fq in response.facet_queries if fq.label == 'object_closure:"HP:0000152"'][0]
    assert hp152.count > 20


def test_association_counts_for_disease():
    si = SolrImplementation()
    association_counts = si.get_association_counts(entity="MONDO:0007947")
    assert association_counts
    assert len(association_counts.items) > 0

    causal_genes = [ac for ac in association_counts.items if ac.category == "biolink:CausalGeneToDiseaseAssociation"][0]
    assert causal_genes.label == "Causal Gene"

    disease_phenotype = [
        ac for ac in association_counts.items if ac.category == "biolink:DiseaseToPhenotypicFeatureAssociation"
    ][0]
    assert disease_phenotype.label == "Disease to Phenotype"


def test_association_counts_for_eds():
    si = SolrImplementation()
    association_counts = si.get_association_counts(entity="MONDO:0020066")
    assert association_counts
    assert len(association_counts.items) > 0


def test_association_counts_for_phenotype():
    si = SolrImplementation()
    association_counts = si.get_association_counts(entity="HP:0000707")  # HP:0025096 ?
    assert association_counts
    assert len(association_counts.items) > 0

    disease_phenotype = [
        ac for ac in association_counts.items if ac.category == "biolink:DiseaseToPhenotypicFeatureAssociation"
    ][0]
    assert disease_phenotype.label == "Disease to Phenotype"

    gene_phenotype = [
        ac for ac in association_counts.items if ac.category == "biolink:GeneToPhenotypicFeatureAssociation"
    ][0]
    assert gene_phenotype.label == "Gene to Phenotype"


def test_association_table():
    si = SolrImplementation()
    association_results = si.get_association_table(
        "MONDO:0007947", AssociationCategory.DISEASE_TO_PHENOTYPIC_FEATURE_ASSOCIATION
    )
    assert association_results
    assert association_results.total > 5
    assert association_results.items[0].direction == AssociationDirectionEnum.outgoing


@pytest.mark.parametrize(
    "id",
    [
        "MONDO:0007523",  # Ehlers-Danlos syndrome, hypermobility type
        "MONDO:0019391",  # Fanconi anemia
        "MONDO:0018954",  # Loeys-Dietz syndrome
        "MONDO:0011518",  # Wiedemann-Steiner syndrome
        "HGNC:4851",  # HTT
        "HGNC:3603",  # FBN1
        "HP:0001166",  # Arachnodactyly
        "HP:0001631",  # Atrial septal defect
        "UBERON:0000948",  # heart
        "UBERON:0006585",  # vestibular organ
    ],
)
def test_empty_search(id):
    si = SolrImplementation()
    response = si.autocomplete("*:*")
    assert response
    assert response.total > 0

    assert id in [x.id for x in response.items]



import pytest
from monarch_py.implementations.solr.solr_implementation import SolrImplementation

pytestmark = pytest.mark.skipif(
    condition=not SolrImplementation().solr_is_available(),
    reason="Solr is not available",
)


@pytest.mark.parametrize(
    "q, should_return",
    [
        ("down syn", "Down syndrome"),
        ("marf", "Marfan syndrome"),
        ("BRCA", "BRCA1"),
    ],
)
def test_autocomplete(q, should_return):
    si = SolrImplementation()
    response = si.autocomplete(q)
    assert response
    assert response.total > 0

    names = [x.name for x in response.items]
    names.extend([x.symbol for x in response.items if x.symbol])
    print(names)
    assert should_return in names



from fastapi.testclient import TestClient
from httpx import URL
from monarch_py.api.main import app

client = TestClient(app)


def test_root_redirects_to_docs():
    response = client.get("/")
    assert response.status_code == 200
    assert "/v3/docs" in URL(response.url).path


def test_api_redirects_to_docs():
    response = client.get("/api")
    assert response.status_code == 200
    assert "/v3/docs" in URL(response.url).path


def test_docs():
    response = client.get("/v3/docs")
    assert response.status_code == 200
    assert response.headers["content-type"] == "text/html; charset=utf-8"


def test_redoc():
    response = client.get("/v3/redoc")
    assert response.status_code == 200
    assert response.headers["content-type"] == "text/html; charset=utf-8"


def test_releases():
    response = client.get("/v3/api/releases?limit=2")
    assert response.status_code == 200
    releases = [release["version"] for release in response.json()]
    assert [i in ["latest", "previous"] for i in releases]


def test_release_metadata():
    response = client.get("/v3/api/releases?release=latest")
    assert response.status_code == 200
    assert response.json()["version"] == "latest"
    assert response.json()["url"] == "https://data.monarchinitiative.org/monarch-kg/latest/index.html"



import urllib
from typing import List

import pytest
from monarch_py.datamodels.solr import SolrQuery


@pytest.mark.parametrize(
    "query,query_string_parts",
    [
        (SolrQuery(), ["q=*:*"]),
        (SolrQuery(rows=10), ["q=*:*", "rows=10"]),
        (SolrQuery(start=101), ["q=*:*", "start=101"]),
        (SolrQuery(q="marfan"), ["q=marfan"]),
        (SolrQuery(facet_fields=["category"]), ["facet.field=category", "facet=true"]),
        (
            SolrQuery(facet_fields=["category", "taxon"]),
            ["q=*:*", "facet.field=category", "facet.field=taxon", "facet=true"],
        ),
        (
            SolrQuery(
                facet_queries=[
                    'object_closure:"HP:0000924"',
                    'object_closure:"HP:0000119"]',
                ]
            ),
            [
                "q=*:*",
                'facet.query=object_closure:"HP:0000924"',
                'facet.query=object_closure:"HP:0000119"]',
                "facet=true",
            ],
        ),
        (
            SolrQuery(filter_queries=['subject_category:"biolink:Gene"']),
            ['fq=subject_category:"biolink:Gene"'],
        ),
        (
            SolrQuery(q="marfan", query_fields="id^100 name^10 synonym"),
            ["qf=", "defType=edismax"],
        ),  # don't worry about actual value of qf
        (
            SolrQuery(
                q="marfan",
                boost='boost=if(termfreq(category,"biolink:Disease"),10.0,1)',
            ),
            ['boost=if(termfreq(category,"biolink:Disease"),10.0,1)'],
        ),
        (SolrQuery(sort="label asc"), ["sort=label+asc"]),
    ],
)
def test_query(query: SolrQuery, query_string_parts: List[str]):
    query_string = urllib.parse.unquote(query.query_string())
    for part in query_string_parts:
        assert part in query_string



import pytest

from monarch_py.implementations.oak.oak_implementation import OakImplementation


@pytest.mark.skip(reason="This is a long running test")
def test_semsim_compare():
    subject_ids = ["MP:0010771"]
    object_ids = ["HP:0004325"]

    print("Getting oak implementation")
    oak = OakImplementation()
    print("Initializing semsim")
    oak.init_semsim()
    print("Running compare")
    tsps = oak.compare(subject_ids, object_ids)

    assert len(tsps.subject_best_matches) == 1
    assert len(tsps.object_best_matches) == 1


if __name__ == "__main__":
    test_semsim_compare()



import pytest

from monarch_py.datamodels.category_enums import (
    AssociationCategory,
    AssociationPredicate,
)
from monarch_py.datamodels.model import Node
from monarch_py.implementations.solr.solr_query_utils import (
    build_association_counts_query,
    build_association_query,
    build_autocomplete_query,
    build_histopheno_query,
    build_mapping_query,
    build_search_query,
    obsolete_unboost,
    entity_predicate_boost,
    category_boost,
    blank_search_boost,
)
from monarch_py.utils.utils import compare_dicts, dict_diff


@pytest.mark.parametrize(
    "direct, facet_fields, facet_queries",
    [
        (True, None, None),
        (False, None, None),
        (None, ["category", "predicate"], [AssociationCategory.DISEASE_TO_PHENOTYPIC_FEATURE_ASSOCIATION.value]),
        (None, ["category", "predicate"], [AssociationCategory.DISEASE_TO_PHENOTYPIC_FEATURE_ASSOCIATION.value]),
    ],
)
def test_build_association_query(
    direct,
    facet_fields,
    facet_queries,
    association_query_params,
    association_query_direct,
    association_query_indirect,
):
    query = build_association_query(
        **association_query_params,
        direct=direct,
        facet_fields=facet_fields,
        facet_queries=facet_queries,
    ).model_dump()
    expected = association_query_direct if direct else association_query_indirect
    if facet_fields:
        expected["facet_fields"] = facet_fields
    if facet_queries:
        expected["facet_queries"] = facet_queries
    assert compare_dicts(query, expected), f"Query is not as expected. Difference: {dict_diff(query, expected)}"


def test_build_association_multiple_categories():
    query = build_association_query(
        category=[
            AssociationCategory.CAUSAL_GENE_TO_DISEASE_ASSOCIATION.value,
            AssociationCategory.DISEASE_TO_PHENOTYPIC_FEATURE_ASSOCIATION.value,
        ]
    )
    assert len(query.filter_queries) > 0, "filter_queries is empty"
    category_filter = [fq for fq in query.filter_queries if fq.startswith("category:")][0]
    assert (
        category_filter
        == "category:biolink\\:CausalGeneToDiseaseAssociation OR category:biolink\\:DiseaseToPhenotypicFeatureAssociation"
    ), "multiple category filter is not as expected"


def test_build_association_multiple_predicates():
    query = build_association_query(
        predicate=[AssociationPredicate.HAS_PHENOTYPE.value, AssociationPredicate.EXPRESSED_IN.value]
    )
    assert len(query.filter_queries) > 0, "filter_queries is empty"
    predicate_filter = [fq for fq in query.filter_queries if fq.startswith("predicate:")][0]
    assert predicate_filter == "predicate:biolink\\:has_phenotype OR predicate:biolink\\:expressed_in", (
        "multiple predicate filter is not as expected"
    )


def test_build_association_multiple_entites():
    query = build_association_query(entity=["MONDO:0020121", "HP:0000006"])
    assert len(query.filter_queries) > 0, "filter_queries is empty"
    entity_filter = [fq for fq in query.filter_queries if fq.startswith("subject:")][0]
    assert (
        entity_filter
        == 'subject:"MONDO:0020121" OR subject_closure:"MONDO:0020121" OR object:"MONDO:0020121" OR object_closure:"MONDO:0020121" OR disease_context_qualifier:"MONDO:0020121" OR disease_context_qualifier_closure:"MONDO:0020121" OR subject:"HP:0000006" OR subject_closure:"HP:0000006" OR object:"HP:0000006" OR object_closure:"HP:0000006" OR disease_context_qualifier:"HP:0000006" OR disease_context_qualifier_closure:"HP:0000006"'
    )


def test_build_association_multiple_subjects():
    query = build_association_query(subject=["MONDO:0020121", "MONDO:0007915"])
    assert len(query.filter_queries) > 0, "filter_queries is empty"
    subject_filter = [fq for fq in query.filter_queries if fq.startswith("subject:")][0]
    assert (
        subject_filter
        == 'subject:"MONDO:0020121" OR subject_closure:"MONDO:0020121" OR subject:"MONDO:0007915" OR subject_closure:"MONDO:0007915"'
    ), "multiple subject filter is not as expected"


def test_build_association_multiple_objects():
    query = build_association_query(object=["HP:0000006", "HP:0000007"])
    assert len(query.filter_queries) > 0, "filter_queries is empty"
    object_filter = [fq for fq in query.filter_queries if fq.startswith("object:")][0]
    print(object_filter)
    assert (
        object_filter
        == 'object:"HP:0000006" OR object_closure:"HP:0000006" OR object:"HP:0000007" OR object_closure:"HP:0000007"'
    ), "multiple object filter is not as expected"


def test_build_association_counts_query(association_counts_query, node):
    query = build_association_counts_query(entity=Node(**node).id).model_dump()
    expected = association_counts_query
    assert compare_dicts(query, expected), f"Query is not as expected. Difference: {dict_diff(query, expected)}"


def test_build_histopheno_query(histopheno_query):
    query = build_histopheno_query("MONDO:0020121").model_dump()
    expected = histopheno_query
    assert compare_dicts(query, expected), f"Query is not as expected. Difference: {dict_diff(query, expected)}"


def test_build_search_query(search_query):
    query = build_search_query(q="fanconi").model_dump()
    expected = search_query
    assert compare_dicts(query, expected), f"Query is not as expected. Difference: {dict_diff(query, expected)}"


def test_build_autocomplete_query(autocomplete_query):
    query = build_autocomplete_query(q="fanc").model_dump()
    expected = autocomplete_query
    assert compare_dicts(query, expected), f"Query is not as expected. Difference: {dict_diff(query, expected)}"


def test_build_mappings_query(mapping_query):
    query = build_mapping_query(entity_id=["MONDO:0020121"]).model_dump()
    expected = mapping_query
    assert compare_dicts(query, expected), f"Query is not as expected. Difference: {dict_diff(query, expected)}"


def test_obsolete_unboost():
    boost = obsolete_unboost()
    assert "deprecated" in boost


def test_entity_predicate_boost() -> str:
    boost = entity_predicate_boost([AssociationPredicate.HAS_PHENOTYPE], 2.0)
    assert "has_phenotype_count" in boost
    assert "2.0" in boost


def test_category_boost():
    boost = category_boost("biolink:PhenotypicFeature", 99.0)
    assert "biolink:PhenotypicFeature" in boost
    assert "99.0" in boost


def test_category_boost_with_taxon():
    boost = category_boost("biolink:Gene", 100.0, taxon="NCBITaxon:9606")
    assert "biolink:Gene" in boost
    assert "100.0" in boost
    assert "NCBITaxon:9606" in boost


def test_blank_search_boost():
    boost = blank_search_boost()
    assert boost.startswith("product(")
    assert boost.endswith(")")
    assert 'if(termfreq(id,"MONDO:0007523"),12,1)' in boost




import pytest

@pytest.fixture
def histopheno_query():
    return {'q': '*:*', 'rows': 0, 'start': 0, 'facet': True, 'facet_min_count': 1, 'facet_fields': [], 'facet_queries': ['object_closure:"UPHENO:0002964"', 'object_closure:"UPHENO:0004523"', 'object_closure:"UPHENO:0002764"', 'object_closure:"UPHENO:0002635"', 'object_closure:"UPHENO:0003020"', 'object_closure:"UPHENO:0080362"', 'object_closure:"HP:0001939"', 'object_closure:"UPHENO:0002642"', 'object_closure:"UPHENO:0002833"', 'object_closure:"HP:0002664"', 'object_closure:"UPHENO:0004459"', 'object_closure:"UPHENO:0002948"', 'object_closure:"UPHENO:0003116"', 'object_closure:"UPHENO:0002816"', 'object_closure:"UPHENO:0004536"', 'object_closure:"HP:0000598"', 'object_closure:"UPHENO:0002712"', 'object_closure:"UPHENO:0075949"', 'object_closure:"UPHENO:0049874"', 'object_closure:"UPHENO:0003013"'], 'filter_queries': ['subject:"MONDO:0020121" OR subject_closure:"MONDO:0020121"'], 'facet_mincount': 1, 'query_fields': None, 'def_type': 'edismax', 'q_op': 'AND', 'mm': '100%', 'boost': None, 'sort': None, 'hl': False}



from unittest.mock import patch

import pytest
from monarch_py.datamodels.model import Association, Entity
from monarch_py.implementations.solr.solr_implementation import SolrImplementation
from monarch_py.implementations.solr.solr_query_utils import entity_boost


@pytest.mark.parametrize(
    ("association", "this_entity", "other_entity"),
    [
        (
            Association(
                id="uuid:0000001",
                subject="ENT:0000003",
                subject_category="biolink:testCase",
                subject_label="Test Case 3",
                subject_closure=["ENT:0000001"],
                object="ENT:0000002",
                object_category="biolink:testCase",
                object_label="Test Case 2",
                predicate="biolink:subclass_of",
                knowledge_level="not_provided",
                agent_type="not_provided",
            ),
            Entity(id="ENT:0000002", name="Test Case 2", category="biolink:testCase"),
            Entity(id="ENT:0000003", name="Test Case 3", category="biolink:testCase"),
        ),
        (
            Association(
                id="uuid:0000002",
                object="ENT:0000004",
                object_category="biolink:testCase",
                object_label="Test Case 4",
                object_closure=["ENT:0000001"],
                subject="ENT:0000003",
                subject_category="biolink:testCase",
                subject_label="Test Case 3",
                predicate="biolink:subclass_of",
                knowledge_level="not_provided",
                agent_type="not_provided",
            ),
            Entity(id="ENT:0000003", name="Test Case 3", category="biolink:testCase"),
            Entity(id="ENT:0000004", name="Test Case 4", category="biolink:testCase"),
        ),
    ],
)
def test_get_counterpart_entity(association, this_entity, other_entity):
    """Test that the get_counterpart_entity function returns the correct entity"""
    si = SolrImplementation()
    associated_entity = si._get_counterpart_entity(association, this_entity)
    assert associated_entity == other_entity, (
        f"Associated entity is not as expected. Expected: {other_entity}, got: {associated_entity}"
    )


def test_entity_boost_with_empty_calls_blank_search_boost():
    # mock blank_search_boost() and assert that it's called
    with patch("monarch_py.implementations.solr.solr_query_utils.blank_search_boost") as mock_blank_search_boost:
        mock_blank_search_boost.return_value = ""
        entity_boost(empty_search=True)
        mock_blank_search_boost.assert_called_once()



import pytest
from fastapi.testclient import TestClient
from fastapi import status
from unittest.mock import MagicMock, patch

from monarch_py.api.additional_models import (
    SemsimMetric,
    SemsimSearchGroup,
    SemsimMultiCompareRequest,
    SemsimDirectionality,
)
from monarch_py.api.semsim import router
from monarch_py.datamodels.category_enums import AssociationPredicate, EntityCategory

client = TestClient(router)


@patch("monarch_py.implementations.solr.solr_implementation.SolrImplementation.autocomplete")
def test_autocomplete_params(mock_autocomplete, autocomplete):
    mock_autocomplete.return_value = autocomplete
    client.get(f"/autocomplete?q=heart")
    mock_autocomplete.assert_called_with(
        q="heart",
        category=[EntityCategory.DISEASE, EntityCategory.GENE, EntityCategory.PHENOTYPIC_FEATURE],
        prioritized_predicates=[AssociationPredicate.HAS_PHENOTYPE],
    )


@pytest.mark.parametrize(
    "metric",
    [
        SemsimMetric.ANCESTOR_INFORMATION_CONTENT,
        SemsimMetric.JACCARD_SIMILARITY,
        SemsimMetric.PHENODIGM_SCORE,
    ],
)
@patch("monarch_py.service.semsim_service.SemsimianService.compare")
def test_get_compare(mock_compare, metric):
    mock_compare.return_value = MagicMock()

    subjects = "HP:123,HP:456"
    objects = "HP:789,HP:987"

    response = client.get(f"/compare/{subjects}/{objects}?metric={metric}")

    assert response.status_code == status.HTTP_200_OK
    mock_compare.assert_called_once_with(subjects=["HP:123", "HP:456"], objects=["HP:789", "HP:987"], metric=metric)


@pytest.mark.parametrize(
    "metric",
    [
        SemsimMetric.ANCESTOR_INFORMATION_CONTENT,
        SemsimMetric.JACCARD_SIMILARITY,
        SemsimMetric.PHENODIGM_SCORE,
    ],
)
@patch("monarch_py.service.semsim_service.SemsimianService.compare")
def test_post_compare(mock_compare, metric):
    mock_compare.return_value = MagicMock()

    subjects = ["HP:123", "HP:456"]
    objects = ["HP:789", "HP:987"]

    response = client.post(f"/compare/", json={"subjects": subjects, "objects": objects, "metric": metric.value})

    assert response.status_code == status.HTTP_200_OK
    mock_compare.assert_called_once_with(subjects=subjects, objects=objects, metric=metric)


@patch("monarch_py.service.semsim_service.SemsimianService.search")
@pytest.mark.parametrize(
    "termset, metric",
    [
        ("HP:123,HP:456", SemsimMetric.ANCESTOR_INFORMATION_CONTENT),
        ("HP:123, HP:456", SemsimMetric.JACCARD_SIMILARITY),
        (" HP:123, HP:456 ", SemsimMetric.PHENODIGM_SCORE),
    ],
)
def test_get_search(mock_search, termset: str, metric: SemsimMetric):
    mock_search.return_value = MagicMock()

    group = SemsimSearchGroup.HGNC
    limit = 5

    response = client.get(f"/search/{termset}/{group.value}?metric={metric}&limit={limit}")
    directionality = SemsimDirectionality.BIDIRECTIONAL
    assert response.status_code == status.HTTP_200_OK
    mock_search.assert_called_once_with(
        termset=["HP:123", "HP:456"], prefix=group.name, metric=metric, directionality=directionality, limit=limit
    )


@patch("monarch_py.service.semsim_service.SemsimianService.search")
def test_post_search(mock_search):
    mock_search.return_value = MagicMock()

    termset = ["HP:123", "HP:456"]
    group = SemsimSearchGroup.HGNC
    metric = SemsimMetric.JACCARD_SIMILARITY
    directionality = SemsimDirectionality.BIDIRECTIONAL
    limit = 5

    response = client.post(
        f"/search/",
        json={
            "termset": termset,
            "group": group.value,
            "metric": metric,
            "directionality": directionality,
            "limit": limit,
        },
    )

    assert response.status_code == status.HTTP_200_OK
    mock_search.assert_called_once_with(termset=["HP:123", "HP:456"], prefix=group.name, metric=metric, limit=limit)


@patch("monarch_py.service.semsim_service.SemsimianService.multi_compare")
def test_get_multi_compare(mock_multi_compare):
    mock_multi_compare.return_value = MagicMock()

    subjects = ["HP:123", "HP:456"]
    object_sets = [
        {"id": "something1", "label": "Test Set", "phenotypes": ["HP:789", "HP:101112"]},
        {"id": "something2", "label": "Test Set 2", "phenotypes": ["HP:987", "HP:102223"]},
    ]

    response = client.post(
        f"/multicompare/",
        json={
            "subjects": subjects,
            "object_sets": object_sets,
        },
    )

    assert response.status_code == status.HTTP_200_OK
    mock_multi_compare.assert_called_once_with(SemsimMultiCompareRequest(subjects=subjects, object_sets=object_sets))



import re
from monarch_py.datamodels.model import ExpandedCurie
from monarch_py.service.curie_service import converter

__all__ = [
    "get_expanded_curie",
]


def expansion_patch(url: str):
    """Any short term patches that we need while waiting for upstream fixes"""

    # This fixes MGI urls that take the form of
    # https://identifiers.org/MGI/97486
    # rather than https://identifiers.org/MGI:97486
    pattern = r"/MGI/(\d+)$"
    replacement = r"/MGI:\1"
    url = re.sub(pattern, replacement, url)

    # Fix phenopacket.store URLs to include /phenopackets/ subdirectory and .json extension
    # ID format: phenopacket.store:GENE.FILENAME (dot separator)
    # Pedigree notation in IDs uses dots (e.g., IV.12) but filenames use underscores (IV_12)
    # Input:  .../notebooks/GENE.FILENAME_WITH.DOTS
    # Output: .../notebooks/GENE/phenopackets/FILENAME_WITH_DOTS.json
    phenopacket_match = re.search(r"(.+phenopacket-store/blob/main/notebooks/)([^.]+)\.(.+)$", url)
    if phenopacket_match:
        prefix, gene, filename = phenopacket_match.groups()
        filename = filename.replace(".", "_")  # Convert pedigree dots to underscores
        url = f"{prefix}{gene}/phenopackets/{filename}.json"

    return url


def get_uri(id: str) -> str | None:
    """Returns the URI for the given CURIE, or None if the prefix is unknown."""
    expanded = converter.expand(id)
    if expanded is None:
        return None
    return expansion_patch(expanded)


def get_expanded_curie(id: str) -> ExpandedCurie:
    """Returns the URI for the given CURIE."""
    return ExpandedCurie(id=id, url=get_uri(id))



from monarch_py.datamodels.model import Node
from monarch_py.datamodels.solr import SolrQueryResult
from monarch_py.implementations.solr.solr_parsers import (
    parse_association_counts,
    parse_association_table,
    parse_associations,
    parse_autocomplete,
    parse_entity,
    parse_histopheno,
    parse_mappings,
    parse_search,
)
from monarch_py.utils.utils import dict_diff


def test_parse_associations(association_response, associations):
    association_response["response"]["numFound"] = association_response["response"].pop("num_found")
    solr_response = SolrQueryResult(**association_response)
    parsed = parse_associations(solr_response).model_dump()
    assert parsed == associations, f"Parsed result is not as expected. Difference: {dict_diff(parsed, associations)}"


def test_parse_associations_compact(association_response, associations_compact):
    association_response["response"]["numFound"] = association_response["response"].pop("num_found")
    solr_response = SolrQueryResult(**association_response)
    parsed = parse_associations(solr_response, compact=True).model_dump()
    assert parsed == associations_compact, (
        f"Parsed result is not as expected. Difference: {dict_diff(parsed, associations_compact)}"
    )


def test_parse_association_counts(association_counts_response, association_counts, node):
    association_counts_response["response"]["numFound"] = association_counts_response["response"].pop("num_found")
    solr_response = SolrQueryResult(**association_counts_response)
    parsed = parse_association_counts(solr_response, entity=Node(**node).id).model_dump()
    assert parsed == association_counts, (
        f"Parsed result is not as expected. Difference: {dict_diff(parsed, association_counts)}"
    )


def test_parse_association_table(association_table_response, association_table, node):
    association_table_response["response"]["numFound"] = association_table_response["response"].pop("num_found")
    solr_response = SolrQueryResult(**association_table_response)
    parsed = parse_association_table(solr_response, entity=[Node(**node).id], offset=0, limit=5).model_dump()
    assert parsed == association_table, (
        f"Parsed result is not as expected. Difference: {dict_diff(parsed, association_table)}"
    )


def test_parse_entity(entity_response, node):
    parsed = parse_entity(entity_response).model_dump()
    assert all(parsed[k] == v for k, v in parsed.items() if k in node)


def test_parse_histopheno(histopheno_response, histopheno, node):
    histopheno_response["response"]["numFound"] = histopheno_response["response"].pop("num_found")
    solr_response = SolrQueryResult(**histopheno_response)
    parsed = parse_histopheno(solr_response, subject_closure=Node(**node).id).model_dump()
    assert parsed == histopheno, f"Parsed result is not as expected. Difference: {dict_diff(parsed, histopheno)}"


def test_parse_search(search_response, search):
    search_response["response"]["numFound"] = search_response["response"].pop("num_found")
    solr_response = SolrQueryResult(**search_response)
    parsed = parse_search(solr_response).model_dump()
    # assert that top level keys are the same
    assert set(parsed.keys()) == set(search.keys()), (
        f"Parsed result keys are not as expected. Difference: {dict_diff(parsed, search)}"
    )
    # compare the first document (parsed.items[0]), assert that all of the keys in expected search (search.items[0])
    for key in search["items"][0].keys():
        assert key in parsed["items"][0], f"Key {key} not found in parsed result."


def test_parse_autocomplete(autocomplete_response, autocomplete):
    autocomplete_response["response"]["numFound"] = autocomplete_response["response"].pop("num_found")
    solr_response = SolrQueryResult(**autocomplete_response)
    parsed = parse_autocomplete(solr_response).model_dump()
    # assert that top level keys are the same
    assert set(parsed.keys()) == set(autocomplete.keys()), (
        f"Parsed result keys are not as expected. Difference: {dict_diff(parsed, autocomplete)}"
    )
    # compare the first document (parsed.items[0]), assert that all of the keys in expected autocomplete (autocomplete.items[0])
    for key in autocomplete["items"][0].keys():
        assert key in parsed["items"][0], f"Key {key} not found in parsed result."


def test_parse_mappings(mapping_response, mappings):
    mapping_response["response"]["numFound"] = mapping_response["response"].pop("num_found")
    solr_response = SolrQueryResult(**mapping_response)
    parsed = parse_mappings(solr_response).model_dump()
    assert parsed == mappings, f"Parsed result is not as expected. Difference: {dict_diff(parsed, mappings)}"



import pytest
from monarch_py.implementations.solr.solr_implementation import SolrImplementation
from monarch_py.implementations.spacy.spacy_implementation import SpacyImplementation


@pytest.fixture
def spacy_instance():
    spacy = SpacyImplementation()
    spacy.init_spacy(grounding_implementation=SolrImplementation())
    return spacy


def test_spacy_positions(spacy_instance):
    entity = spacy_instance.nlp("the brain").ents[0]
    entity.start = 4
    entity.end = 9


def test_spacy_positions_with_two_entities(spacy_instance):
    entities = spacy_instance.nlp("the brain and the heart").ents
    assert len(entities) == 2
    assert entities[0].start_char == 4
    assert entities[0].end_char == 9
    assert entities[1].start_char == 18
    assert entities[1].end_char == 23


@pytest.mark.skipif(
    condition=not SolrImplementation().solr_is_available(),
    reason="Solr is not available",
)
def test_entity_positions(spacy_instance):
    entities = spacy_instance.get_annotated_entities("the brain")
    assert entities
    assert len(entities) == 2
    assert entities[0].text == "the "
    assert entities[1].text == "brain"
    assert entities[1].start == 4
    assert entities[1].end == 9


@pytest.mark.skipif(
    condition=not SolrImplementation().solr_is_available(),
    reason="Solr is not available",
)
def test_annotated_entities(spacy_instance):
    entities = spacy_instance.get_annotated_entities("let me tell you about Marfan syndrome, ok?")
    assert entities
    assert len(entities) == 3
    assert entities[0].text == "let me tell you about "
    assert entities[1].text == "Marfan syndrome"
    assert entities[2].text == ", ok?"


@pytest.mark.skipif(
    condition=not SolrImplementation().solr_is_available(),
    reason="Solr is not available",
)
def test_non_entity_text_is_returned_between_entities(spacy_instance):
    entities = spacy_instance.get_annotated_entities(
        "let me tell you about Marfan syndrome and Ehlers Danlos syndrome, ok?"
    )
    assert entities
    assert len(entities) > 1
    assert entities[0].text == "let me tell you about "
    assert entities[1].text == "Marfan syndrome"
    assert entities[2].text == " and "
    assert entities[3].text == "Ehlers Danlos syndrome"
    assert entities[4].text == ", ok?"


# This test is intended to act as guard rails and documentation of entity recognition quality, if we collect
# terms that work and do not work here, we can eventually test a full list against alternate models
@pytest.mark.parametrize(
    "text",
    [
        "Marfan syndrome",
        "Ehlers-Danlos syndrome",
        # fails: "Ehlers Danlos syndrome",
        # fails: "Ehlers Danlos",
        "Loeys-Dietz syndrome",
        "connective tissue disorder",
    ],
)
def test_entity_recognition(spacy_instance, text):
    entities = spacy_instance.nlp(text).ents
    assert [entity.text for entity in entities] == [text]



import pytest
from monarch_py.implementations.sql.sql_implementation import SQLImplementation

pytestmark = pytest.mark.skip("Need to rewrite SQL tests")


def test_associations():
    data = SQLImplementation()
    response = data.get_associations()
    assert response
    assert response.total > 100000
    assert len(response.items) == 20


def test_association_page_limit():
    data = SQLImplementation()
    response = data.get_associations(limit=100)
    assert len(response.items) == 100


def test_association_category():
    data = SQLImplementation()
    response = data.get_associations(category="biolink:GeneToDiseaseAssociation")
    assert response
    assert response.total > 6000
    assert "biolink:GeneToDiseaseAssociation" in response.items[0].category


def test_association_predicate():
    data = SQLImplementation()
    response = data.get_associations(predicate="biolink:has_phenotype")
    assert response
    assert response.total > 600000
    assert "biolink:has_phenotype" in response.items[0].predicate


def test_subject():
    data = SQLImplementation()
    response = data.get_associations(subject="MONDO:0007947")
    assert response
    assert response.total > 50
    assert response.items[0].subject == "MONDO:0007947"


def test_object():
    data = SQLImplementation()
    response = data.get_associations(object="MONDO:0007947")
    assert response
    assert response.total > 1
    assert response.items[0].object == "MONDO:0007947"


def test_entity():
    data = SQLImplementation()
    response = data.get_associations(entity="MONDO:0007947")
    assert response
    assert response.total > 50
    for association in response.items:
        assert association.subject == "MONDO:0007947" or association.object == "MONDO:0007947"


def test_between():
    data = SQLImplementation()
    response = data.get_associations(between="MONDO:0007947,HP:0000098")
    assert response
    assert response.total > 0
    for association in response.items:
        assert association.subject == "MONDO:0007947" and association.object == "HP:0000098"


def test_between_reversed():
    data = SQLImplementation()
    response = data.get_associations(between="HP:0000098,MONDO:0007947")
    assert response
    assert response.total > 0
    for association in response.items:
        assert association.subject == "MONDO:0007947" and association.object == "HP:0000098"


def test_associations_by_type():
    data = SQLImplementation()
    response = data.get_associations(entity="MONDO:0007947", category="biolink:DiseaseToPhenotypicFeatureAssociation")

    assert response
    assert response.total > 60
    assert response.items[0].subject == "MONDO:0007947"



import pytest
from monarch_py.implementations.solr.solr_implementation import SolrImplementation

pytestmark = pytest.mark.skipif(
    condition=not SolrImplementation().solr_is_available(),
    reason="Solr is not available",
)


def test_entity():
    si = SolrImplementation()
    entity = si.get_entity("MONDO:0007947", extra=False)
    assert entity
    assert entity.name == "Marfan syndrome"


def test_hierarchy_limit():
    si = SolrImplementation()
    entity = si.get_entity("MONDO:0700096", extra=True)
    assert entity.node_hierarchy
    assert len(entity.node_hierarchy.sub_classes) > 20



import pytest
from unittest.mock import MagicMock

from monarch_py.datamodels.model import (
    AssociationResults,
    HistoPheno,
    Node,
    SearchResults,
    ExpandedCurie,
)
from monarch_py.utils.utils import (
    compare_dicts,
    dict_factory,
    dict_diff,
    escape,
    get_links_for_field,
    get_provided_by_link,
    strip_json,
)
from monarch_py.utils.format_utils import get_headers_from_obj, to_json, to_tsv, to_yaml

### Test basic utility methods ###


def test_strip_json():
    doc = {"a": 1, "b": 2, "c": 3}
    assert strip_json(doc, "a", "b") == {"c": 3}


def test_strip_json_missing():
    doc = {"a": 1, "b": 2, "c": 3}
    assert strip_json(doc, "d") == {"a": 1, "b": 2, "c": 3}


def test_escape():
    assert escape("HP:123") == r"HP\:123"


def test_dict_factory():
    cursor = MagicMock()
    cursor.description = [("a",), ("b",), ("c",)]
    row = (1, 2, 3)
    assert dict_factory(cursor, row) == {"a": 1, "b": 2, "c": 3}


@pytest.mark.parametrize(
    "dict1,dict2",
    [
        ({"a": 1, "b": 2}, {"a": 1, "b": 2}),
        ({"a": 1, "b": 2}, {"b": 2, "a": 1}),
    ],
)
def test_compare_dicts(dict1, dict2):
    assert compare_dicts(dict1, dict2) is True


def test_compare_dicts_false():
    assert compare_dicts({"a": 1, "b": 2}, {"a": 1, "b": 3}) is False


@pytest.mark.parametrize(
    "d1, d2, expected",
    [
        ({"a": 1, "b": 2}, {"a": 1, "b": 2}, {}),
        ({"a": 1, "b": 2}, {"b": 2, "a": 1}, {}),
        ({"a": 1, "b": 2}, {"a": 1, "b": 3}, {"Dict-1-b": 2, "Dict-2-b": 3}),
    ],
)
def test_dict_diff(d1, d2, expected):
    assert dict_diff(d1, d2) == expected


### Test URL fetching methods ###


def test_get_links_for_field():
    field = ["PMID:123", "PMID:456"]
    assert get_links_for_field(field) == [
        ExpandedCurie(id="PMID:123", url="http://identifiers.org/pubmed/123"),
        ExpandedCurie(id="PMID:456", url="http://identifiers.org/pubmed/456"),
    ]


def test_get_provided_by_link():
    assert get_provided_by_link("alliance_gene_nodes") == ExpandedCurie(
        id="alliance_gene", url="https://monarch-initiative.github.io/monarch-ingest/Sources/alliance/#gene"
    )


### Test output conversion methods ###


@pytest.mark.parametrize(
    "obj, expected",
    [
        ("node", "node_headers"),
        ("search", "search_headers"),
        ("histopheno", "histobin_headers"),
        ("associations", "association_headers"),
    ],
)
def test_get_headers_from_obj(obj, expected, request):
    if obj == "node":
        node = request.getfixturevalue(obj)
        obj = Node(**node)
    elif obj == "search":
        search = request.getfixturevalue(obj)
        obj = SearchResults(**search)
    elif obj == "histopheno":
        histopheno = request.getfixturevalue(obj)
        obj = HistoPheno(**histopheno)
    elif obj == "associations":
        associations = request.getfixturevalue(obj)
        obj = AssociationResults(**associations)

    expected = request.getfixturevalue(expected)
    headers = get_headers_from_obj(obj)
    # Allow for the expected headers to be a subset of the actual headers,
    # so that the data model being ahead of new fields the Solr index won't
    # break the tests
    for header in expected:
        assert header in headers


@pytest.mark.skip(reason="It's broken and I'm dumb")
@pytest.mark.parametrize(
    "obj, expected, format",
    [
        ("node", "node_json", "json"),
        ("node", "node_tsv", "tsv"),
        ("node", "node_yaml", "yaml"),
    ],
)
def test_convert_to_tsv(obj, expected, format, request):
    obj = request.getfixturevalue(obj)
    node = Node(**obj)
    if format == "json":
        result = to_json(node, print_output=False)
    elif format == "tsv":
        result = to_tsv(node, print_output=False)
    elif format == "yaml":
        result = to_yaml(node, print_output=False)
    expected = request.getfixturevalue(expected)
    assert result == expected  # type: ignore



import pytest
from monarch_py.implementations.sql.sql_implementation import SQLImplementation

pytestmark = pytest.mark.skip("all tests still WIP")


def test_entity():
    data = SQLImplementation()
    entity = data.get_entity("MONDO:0007947")
    assert entity
    assert entity.name == "Marfan syndrome"



from fastapi.testclient import TestClient
from unittest.mock import MagicMock, patch
from monarch_py.api.text_annotation import router

client = TestClient(router)


@patch("monarch_py.implementations.spacy.spacy_implementation.SpacyImplementation.annotate_text")
def test_get_annotate(mock_annotate):
    mock_annotate.return_value = '<span class="this-is-just-mocked">Ehlers-Danlos syndrome</span>'
    response = client.get("/annotate?text=Ehlers-Danlos syndrome")
    assert response.status_code == 200
    mock_annotate.assert_called_with("Ehlers-Danlos syndrome")


@patch("monarch_py.implementations.spacy.spacy_implementation.SpacyImplementation.annotate_text")
def test_post_annotate(mock_annotate):
    mock_annotate.return_value = '<span class="this-is-just-mocked">Ehlers-Danlos syndrome</span>'
    response = client.post("/annotate", json={"content": "Ehlers-Danlos syndrome"})
    assert response.status_code == 200
    mock_annotate.assert_called_with("Ehlers-Danlos syndrome")


@patch("monarch_py.implementations.spacy.spacy_implementation.SpacyImplementation.get_annotated_entities")
def test_get_annotate_entities(mock_annotate):
    mock_annotate.return_value = MagicMock()
    response = client.get("/annotate/entities?text=Ehlers-Danlos syndrome")
    assert response.status_code == 200
    mock_annotate.assert_called_with("Ehlers-Danlos syndrome")


@patch("monarch_py.implementations.spacy.spacy_implementation.SpacyImplementation.get_annotated_entities")
def test_post_annotate_entities(mock_annotate):
    mock_annotate.return_value = MagicMock()
    response = client.post("/annotate/entities", json={"content": "Ehlers-Danlos syndrome"})
    assert response.status_code == 200
    mock_annotate.assert_called_with("Ehlers-Danlos syndrome")




import pytest

@pytest.fixture
def histopheno():
    return {'id': 'MONDO:0020121', 'items': [{'label': 'musculature', 'count': 2135, 'id': 'UPHENO:0002816'}, {'label': 'nervous_system', 'count': 1042, 'id': 'UPHENO:0004523'}, {'label': 'head_neck', 'count': 608, 'id': 'UPHENO:0002764'}, {'label': 'skeletal_system', 'count': 524, 'id': 'UPHENO:0002964'}, {'label': 'eye', 'count': 300, 'id': 'UPHENO:0003020'}, {'label': 'metabolism_homeostasis', 'count': 229, 'id': 'HP:0001939'}, {'label': 'blood', 'count': 204, 'id': 'UPHENO:0004459'}, {'label': 'digestive_system', 'count': 188, 'id': 'UPHENO:0002833'}, {'label': 'connective_tissue', 'count': 187, 'id': 'UPHENO:0002712'}, {'label': 'respiratory', 'count': 161, 'id': 'UPHENO:0004536'}, {'label': 'integument', 'count': 51, 'id': 'UPHENO:0002635'}, {'label': 'genitourinary_system', 'count': 48, 'id': 'UPHENO:0002642'}, {'label': 'endocrine', 'count': 40, 'id': 'UPHENO:0003116'}, {'label': 'growth', 'count': 33, 'id': 'UPHENO:0049874'}, {'label': 'immune_system', 'count': 28, 'id': 'UPHENO:0002948'}, {'label': 'ear', 'count': 23, 'id': 'HP:0000598'}, {'label': 'prenatal_or_birth', 'count': 22, 'id': 'UPHENO:0075949'}, {'label': 'neoplasm', 'count': 7, 'id': 'HP:0002664'}, {'label': 'breast', 'count': 1, 'id': 'UPHENO:0003013'}, {'label': 'cardiovascular_system', 'count': 0, 'id': 'UPHENO:0080362'}]}



########################################################################
# This file is used to import pytest fixtures from the tests directory #
########################################################################

from monarch_py.datamodels.model import AssociationResults, HistoPheno, Node, SearchResults


def test_autocomplete(autocomplete):
    autocomplete = SearchResults(**autocomplete)
    assert autocomplete.total != 0


def test_search(search):
    search = SearchResults(**search)
    assert search.total != 0


def test_histopheno(histopheno):
    histopheno = HistoPheno(**histopheno)
    assert histopheno.id == "MONDO:0020121"


def test_node(node):
    node = Node(**node)
    assert node.id == "MONDO:0020121"


def test_node_associations(associations):
    association_results = AssociationResults(**associations)
    assert association_results.total != 0




import pytest

@pytest.fixture
def association_headers():
    return ['id', 'category', 'subject', 'original_subject', 'subject_namespace', 'subject_category', 'subject_closure', 'subject_label', 'subject_closure_label', 'subject_taxon', 'subject_taxon_label', 'predicate', 'original_predicate', 'object', 'original_object', 'object_namespace', 'object_category', 'object_closure', 'object_label', 'object_closure_label', 'object_taxon', 'object_taxon_label', 'primary_knowledge_source', 'aggregator_knowledge_source', 'negated', 'pathway', 'evidence_count', 'knowledge_level', 'agent_type', 'has_evidence', 'has_evidence_links', 'has_count', 'has_total', 'has_percentage', 'has_quotient', 'grouping_key', 'provided_by', 'provided_by_link', 'publications', 'publications_links', 'frequency_qualifier', 'onset_qualifier', 'sex_qualifier', 'stage_qualifier', 'qualifiers', 'qualifiers_label', 'qualifiers_namespace', 'qualifiers_category', 'qualifier', 'qualifier_label', 'qualifier_namespace', 'qualifier_category', 'frequency_qualifier_label', 'frequency_qualifier_namespace', 'frequency_qualifier_category', 'onset_qualifier_label', 'onset_qualifier_namespace', 'onset_qualifier_category', 'sex_qualifier_label', 'sex_qualifier_namespace', 'sex_qualifier_category', 'stage_qualifier_label', 'stage_qualifier_namespace', 'stage_qualifier_category', 'disease_context_qualifier', 'disease_context_qualifier_label', 'disease_context_qualifier_namespace', 'disease_context_qualifier_category', 'disease_context_qualifier_closure', 'disease_context_qualifier_closure_label', 'species_context_qualifier', 'species_context_qualifier_label', 'species_context_qualifier_namespace', 'species_context_qualifier_category', 'subject_specialization_qualifier', 'subject_specialization_qualifier_label', 'subject_specialization_qualifier_namespace', 'subject_specialization_qualifier_category', 'subject_specialization_qualifier_closure', 'subject_specialization_qualifier_closure_label', 'object_specialization_qualifier', 'object_specialization_qualifier_label', 'object_specialization_qualifier_namespace', 'object_specialization_qualifier_category', 'object_specialization_qualifier_closure', 'object_specialization_qualifier_closure_label', 'highlighting']

@pytest.fixture
def histobin_headers():
    return ['label', 'count', 'id']

@pytest.fixture
def node_headers():
    return ['id', 'category', 'name', 'full_name', 'deprecated', 'description', 'xref', 'provided_by', 'in_taxon', 'in_taxon_label', 'symbol', 'synonym', 'broad_synonym', 'exact_synonym', 'narrow_synonym', 'related_synonym', 'subsets', 'uri', 'iri', 'namespace', 'has_phenotype', 'has_phenotype_label', 'has_phenotype_closure', 'has_phenotype_closure_label', 'has_phenotype_count', 'has_descendant', 'has_descendant_label', 'has_descendant_count', 'inheritance', 'causal_gene', 'causes_disease', 'mappings', 'external_links', 'provided_by_link', 'association_counts', 'node_hierarchy']

@pytest.fixture
def search_headers():
    return ['id', 'category', 'name', 'full_name', 'deprecated', 'description', 'xref', 'provided_by', 'in_taxon', 'in_taxon_label', 'symbol', 'synonym', 'broad_synonym', 'exact_synonym', 'narrow_synonym', 'related_synonym', 'subsets', 'uri', 'iri', 'namespace', 'has_phenotype', 'has_phenotype_label', 'has_phenotype_closure', 'has_phenotype_closure_label', 'has_phenotype_count', 'has_descendant', 'has_descendant_label', 'has_descendant_count', 'score', 'highlighting']



import pytest
from monarch_py.datamodels.model import AssociationTypeMapping
from monarch_py.utils.association_type_utils import (
    get_solr_query_fragment,
    get_sql_query_fragment,
    parse_query_string_for_category,
)


@pytest.fixture()
def basic_mapping():
    return AssociationTypeMapping(
        subject_label="Genes",
        object_label="Phenotypes",
        category="biolink:GeneToPhenotypeAssociation",
    )


def test_solr_basic_mapping(basic_mapping):
    query_fragment = get_solr_query_fragment(basic_mapping)
    assert query_fragment == 'category:"biolink:GeneToPhenotypeAssociation"'


def test_sql_basic_mapping(basic_mapping):
    query_fragment = get_sql_query_fragment(basic_mapping)
    assert query_fragment == 'category = "biolink:GeneToPhenotypeAssociation"'


def test_parse_association_type_query_string_single_category():
    query_string = 'category:"biolink:GeneToPhenotypeAssociation"'
    category = parse_query_string_for_category(query_string)
    assert category == "biolink:GeneToPhenotypeAssociation"


def test_parse_association_type_query_string_multiple_categories():
    query_string = 'category:"biolink:GeneToDiseaseAssociation" AND (predicate:"biolink:gene_associated_with_condition" OR predicate:"biolink:contributes_to")'
    category = parse_query_string_for_category(query_string)
    assert category == "biolink:GeneToDiseaseAssociation"



from unittest.mock import patch

from monarch_py.implementations.solr.solr_implementation import SolrImplementation


def test_get_counterpart_entities_limit():
    # Make sure that we don't accidentally end up with the default limit again
    with patch.object(SolrImplementation, "get_associations") as mock_get_associations:
        si = SolrImplementation()
        si.get_counterpart_entities("MONDO:0007947")
        assert mock_get_associations.called
        assert mock_get_associations.call_args_list[0].kwargs["limit"] == 1000



import urllib
from unittest.mock import MagicMock, patch

from fastapi.testclient import TestClient
from httpx import Response

from monarch_py.api.search import router
from monarch_py.datamodels.category_enums import EntityCategory
from monarch_py.datamodels.model import SearchResults

client = TestClient(router)


def test_search(search):
    with patch.object(
        client, "get", MagicMock(return_value=Response(200, json=search, headers={"content-type": "application/json"}))
    ):
        response = client.get("/search?q=heart")
    assert response.status_code == 200
    assert response.headers["content-type"] == "application/json"
    assert response.json() == search


@patch("monarch_py.implementations.solr.solr_implementation.SolrImplementation.search")
def test_search_params(mock_search, search):
    mock_search.return_value = SearchResults(**search)
    params = {
        "q": "heart",
        "category": [EntityCategory.DISEASE.value, EntityCategory.PHENOTYPIC_FEATURE.value],
        "in_taxon_label": ["NCBITaxon:9606", "NCBITaxon:10090"],
        "offset": 0,
        "limit": 20,
    }

    query_string = urllib.parse.urlencode(params, doseq=True)
    client.get(f"/search?{query_string}")
    search_params = {**params, "facet_fields": ["category", "in_taxon_label"], "highlighting": True}
    search_params["category"] = [EntityCategory(c) for c in search_params["category"]]
    search_params["category"] = [EntityCategory.DISEASE, EntityCategory.PHENOTYPIC_FEATURE]
    mock_search.assert_called_with(**search_params)


@patch("monarch_py.implementations.solr.solr_implementation.SolrImplementation.autocomplete")
def test_autocomplete_params(mock_autocomplete, autocomplete):
    mock_autocomplete.return_value = autocomplete
    client.get(f"/autocomplete?q=heart")
    mock_autocomplete.assert_called_with(q="heart")



from monarch_py.service.semsim_service import SemsimianService

import pytest


@pytest.fixture
def semsim_service():
    return SemsimianService(semsim_server_host="localhost", semsim_server_port=8080, entity_implementation=None)


@pytest.mark.parametrize(
    "data, expected",
    [
        ({"some_dict": {"a": 1, "b": "NaN", "c": 3}}, {"some_dict": {"a": 1, "b": None, "c": 3}}),
        ({"some_dict": {"a": 1, "b": "two", "c": 3}}, {"some_dict": {"a": 1, "b": "two", "c": 3}}),
    ],
)
def test_convert_nans(semsim_service, data, expected):
    result = semsim_service._convert_nans(data)
    assert result == expected




import pytest

@pytest.fixture
def histopheno_response():
    return {'responseHeader': {'QTime': 1, 'params': {'facet.query': ['object_closure:"UPHENO:0002964"', 'object_closure:"UPHENO:0004523"', 'object_closure:"UPHENO:0002764"', 'object_closure:"UPHENO:0002635"', 'object_closure:"UPHENO:0003020"', 'object_closure:"UPHENO:0080362"', 'object_closure:"HP:0001939"', 'object_closure:"UPHENO:0002642"', 'object_closure:"UPHENO:0002833"', 'object_closure:"HP:0002664"', 'object_closure:"UPHENO:0004459"', 'object_closure:"UPHENO:0002948"', 'object_closure:"UPHENO:0003116"', 'object_closure:"UPHENO:0002816"', 'object_closure:"UPHENO:0004536"', 'object_closure:"HP:0000598"', 'object_closure:"UPHENO:0002712"', 'object_closure:"UPHENO:0075949"', 'object_closure:"UPHENO:0049874"', 'object_closure:"UPHENO:0003013"'], 'mm': '100%', 'q': '*:*', 'defType': 'edismax', 'facet_min_count': '1', 'hl': 'false', 'start': '0', 'q.op': 'AND', 'fq': 'subject:"MONDO:0020121" OR subject_closure:"MONDO:0020121"', 'facet.mincount': '1', 'rows': '0', 'facet': 'true'}}, 'response': {'num_found': 4655, 'start': 0, 'docs': []}, 'facet_counts': {'facet_fields': {}, 'facet_queries': {'object_closure:"UPHENO:0002964"': 524, 'object_closure:"UPHENO:0004523"': 1042, 'object_closure:"UPHENO:0002764"': 608, 'object_closure:"UPHENO:0002635"': 51, 'object_closure:"UPHENO:0003020"': 300, 'object_closure:"UPHENO:0080362"': 0, 'object_closure:"HP:0001939"': 229, 'object_closure:"UPHENO:0002642"': 48, 'object_closure:"UPHENO:0002833"': 188, 'object_closure:"HP:0002664"': 7, 'object_closure:"UPHENO:0004459"': 204, 'object_closure:"UPHENO:0002948"': 28, 'object_closure:"UPHENO:0003116"': 40, 'object_closure:"UPHENO:0002816"': 2135, 'object_closure:"UPHENO:0004536"': 161, 'object_closure:"HP:0000598"': 23, 'object_closure:"UPHENO:0002712"': 187, 'object_closure:"UPHENO:0075949"': 22, 'object_closure:"UPHENO:0049874"': 33, 'object_closure:"UPHENO:0003013"': 1}}, 'highlighting': {}}




import pytest

@pytest.fixture
def autocomplete_query():
    return {'q': 'fanc', 'rows': 10, 'start': 0, 'facet': True, 'facet_min_count': 1, 'facet_fields': [], 'facet_queries': [], 'filter_queries': [], 'facet_mincount': 1, 'query_fields': 'id^100 name^10 name_t^5 name_ac symbol^10 symbol_t^5 symbol_ac synonym synonym_t synonym_ac description_t xref', 'def_type': 'edismax', 'q_op': 'AND', 'mm': '100%', 'boost': 'product(if(termfreq(category,"biolink:PhenotypicFeature"),1.1,1),if(termfreq(category,"biolink:Disease"),1.3,1),if(and(termfreq(in_taxon,"NCBITaxon:9606"),termfreq(category,"biolink:Gene")),1.1,1),if(termfreq(deprecated,"true"),0.1,1))', 'sort': None, 'hl': False}



from monarch_py.service.curie_service import converter
import pytest


@pytest.mark.parametrize(
    ("curie", "expected"),
    [
        (
            "FB:FBgn0000008",
            "identifiers.org/flybase/FBgn0000008",
        ),  # this is a little odd, fb vs. flybase, but both resolve
        ("FlyBase:FBgn0000008", "identifiers.org/fb/FBgn0000008"),
        ("MONDO:0005737", "purl.obolibrary.org/obo/MONDO_0005737"),
        ("NCBIGene:1017", "identifiers.org/ncbigene/1017"),
        ("ZFIN:ZDB-GENE-980526-166", "identifiers.org/zfin/ZDB-GENE-980526-166"),
        ("MGI:1918910", "identifiers.org/MGI/1918910"),
        ("HP:0000001", "purl.obolibrary.org/obo/HP_0000001"),
        ("GO:0000001", "purl.obolibrary.org/obo/GO_0000001"),
        ("CL:0000001", "purl.obolibrary.org/obo/CL_0000001"),
        ("WB:WBGene00000001", "identifiers.org/wb/WBGene00000001"),
        ("SGD:S000000001", "identifiers.org/sgd/S000000001"),
        ("RGD:1", "identifiers.org/rgd/1"),
        ("HGNC:5", "identifiers.org/hgnc/5"),
        ### These prefixes are currently broken in the curie service
        ("ENSEMBL:ENSG00000157764", "identifiers.org/ensembl/ENSG00000157764"),
        ("OMIM:613647", "identifiers.org/mim/613647"),
        ("UBERON:0000001", "purl.obolibrary.org/obo/UBERON_0000001"),
        ("WormBase:WBGene00000001", "identifiers.org/wb/WBGene00000001"),
    ],
)
def test_curie_expansion(curie, expected):
    expanded_curie = converter.expand(curie)
    # print(expanded_curie, expected)
    assert expected in expanded_curie



from __future__ import annotations 

import re
import sys
from datetime import (
    date,
    datetime,
    time
)
from decimal import Decimal 
from enum import Enum 
from typing import (
    Any,
    ClassVar,
    Literal,
    Optional,
    Union
)

from pydantic import (
    BaseModel,
    ConfigDict,
    Field,
    RootModel,
    field_validator
)


metamodel_version = "None"
version = "None"


class ConfiguredBaseModel(BaseModel):
    model_config = ConfigDict(
        validate_assignment = True,
        validate_default = True,
        extra = "allow",
        arbitrary_types_allowed = True,
        use_enum_values = True,
        strict = False,
    )
    pass




class LinkMLMeta(RootModel):
    root: dict[str, Any] = {}
    model_config = ConfigDict(frozen=True)

    def __getattr__(self, key:str):
        return getattr(self.root, key)

    def __getitem__(self, key:str):
        return self.root[key]

    def __setitem__(self, key:str, value):
        self.root[key] = value

    def __contains__(self, key:str) -> bool:
        return key in self.root


linkml_meta = None

class AssociationDirectionEnum(str, Enum):
    """
    The directionality of an association as it relates to a specified entity, with edges being categorized as incoming or outgoing
    """
    incoming = "incoming"
    """
    An association for which a specified entity is the object or part of the object closure
    """
    outgoing = "outgoing"
    """
    An association for which a specified entity is the subject or part of the subject closure
    """



class PairwiseSimilarity(ConfiguredBaseModel):
    """
    Abstract grouping for representing individual pairwise similarities
    """
    pass


class TermPairwiseSimilarity(PairwiseSimilarity):
    """
    A simple pairwise similarity between two atomic concepts/terms
    """
    subject_id: str = Field(default=...)
    subject_label: Optional[str] = Field(default=None, description="""The name of the subject entity""")
    subject_source: Optional[str] = Field(default=None, description="""the source for the first entity""")
    object_id: str = Field(default=...)
    object_label: Optional[str] = Field(default=None, description="""The name of the object entity""")
    object_source: Optional[str] = Field(default=None, description="""the source for the second entity""")
    ancestor_id: Optional[str] = Field(default=None, description="""the most recent common ancestor of the two compared entities. If there are multiple MRCAs then the most informative one is selected""")
    ancestor_label: Optional[str] = Field(default=None, description="""the name or label of the ancestor concept""")
    ancestor_source: Optional[str] = Field(default=None)
    object_information_content: Optional[float] = Field(default=None, description="""The IC of the object""")
    subject_information_content: Optional[float] = Field(default=None, description="""The IC of the subject""")
    ancestor_information_content: Optional[float] = Field(default=None, description="""The IC of the object""")
    jaccard_similarity: Optional[float] = Field(default=None, description="""The number of concepts in the intersection divided by the number in the union""")
    cosine_similarity: Optional[float] = Field(default=None, description="""the dot product of two node embeddings divided by the product of their lengths""")
    dice_similarity: Optional[float] = Field(default=None)
    phenodigm_score: Optional[float] = Field(default=None, description="""the geometric mean of the jaccard similarity and the information content""")


class TermSetPairwiseSimilarity(PairwiseSimilarity):
    """
    A simple pairwise similarity between two sets of concepts/terms
    """
    subject_termset: Optional[dict[str, Union[str, TermInfo]]] = Field(default=None)
    object_termset: Optional[dict[str, Union[str, TermInfo]]] = Field(default=None)
    subject_best_matches: Optional[dict[str, BestMatch]] = Field(default=None)
    object_best_matches: Optional[dict[str, BestMatch]] = Field(default=None)
    average_score: Optional[float] = Field(default=None)
    best_score: Optional[float] = Field(default=None)
    metric: Optional[str] = Field(default=None)


class TermInfo(ConfiguredBaseModel):
    id: str = Field(default=...)
    label: Optional[str] = Field(default=None)


class BestMatch(ConfiguredBaseModel):
    match_source: str = Field(default=...)
    match_source_label: Optional[str] = Field(default=None)
    match_target: Optional[str] = Field(default=None, description="""the entity matches""")
    match_target_label: Optional[str] = Field(default=None)
    score: float = Field(default=...)
    match_subsumer: Optional[str] = Field(default=None)
    match_subsumer_label: Optional[str] = Field(default=None)
    similarity: TermPairwiseSimilarity = Field(default=...)


class SemsimSearchResult(ConfiguredBaseModel):
    subject: Entity = Field(default=...)
    score: Optional[float] = Field(default=None)
    similarity: Optional[TermSetPairwiseSimilarity] = Field(default=None)


class Association(ConfiguredBaseModel):
    id: str = Field(default=...)
    category: Optional[str] = Field(default=None)
    subject: str = Field(default=...)
    original_subject: Optional[str] = Field(default=None)
    subject_namespace: Optional[str] = Field(default=None, description="""The namespace/prefix of the subject entity""")
    subject_category: Optional[str] = Field(default=None, description="""The category of the subject entity""")
    subject_closure: Optional[list[str]] = Field(default=None, description="""Field containing subject id and the ids of all of it's ancestors""")
    subject_label: Optional[str] = Field(default=None, description="""The name of the subject entity""")
    subject_closure_label: Optional[list[str]] = Field(default=None, description="""Field containing subject name and the names of all of it's ancestors""")
    subject_taxon: Optional[str] = Field(default=None)
    subject_taxon_label: Optional[str] = Field(default=None)
    predicate: str = Field(default=...)
    original_predicate: Optional[str] = Field(default=None, description="""used to hold the original relation/predicate that an external knowledge source uses before transformation to match the biolink-model specification.""")
    object: str = Field(default=...)
    original_object: Optional[str] = Field(default=None)
    object_namespace: Optional[str] = Field(default=None, description="""The namespace/prefix of the object entity""")
    object_category: Optional[str] = Field(default=None, description="""The category of the object entity""")
    object_closure: Optional[list[str]] = Field(default=None, description="""Field containing object id and the ids of all of it's ancestors""")
    object_label: Optional[str] = Field(default=None, description="""The name of the object entity""")
    object_closure_label: Optional[list[str]] = Field(default=None, description="""Field containing object name and the names of all of it's ancestors""")
    object_taxon: Optional[str] = Field(default=None)
    object_taxon_label: Optional[str] = Field(default=None)
    primary_knowledge_source: Optional[str] = Field(default=None)
    aggregator_knowledge_source: Optional[list[str]] = Field(default=None)
    negated: Optional[bool] = Field(default=None)
    pathway: Optional[str] = Field(default=None)
    evidence_count: Optional[int] = Field(default=None, description="""count of supporting documents, evidence codes, and sources supplying evidence""")
    knowledge_level: str = Field(default=..., description="""Describes the level of knowledge expressed in a statement, based on the reasoning or analysis methods used to generate the statement, or the scope or specificity of what the statement expresses to be true.""")
    agent_type: str = Field(default=..., description="""Describes the high-level category of agent who originally generated a  statement of knowledge or other type of information.""")
    has_evidence: Optional[list[str]] = Field(default=None)
    has_evidence_links: Optional[list[ExpandedCurie]] = Field(default=None, description="""List of ExpandedCuries with id and url for evidence""")
    has_count: Optional[int] = Field(default=None, description="""count of out of has_total representing a frequency""")
    has_total: Optional[int] = Field(default=None, description="""total, devided by has_count, representing a frequency""")
    has_percentage: Optional[float] = Field(default=None, description="""percentage, which may be calculated from has_count and has_total, as 100 * quotient or provided directly, rounded to the integer level""")
    has_quotient: Optional[float] = Field(default=None, description="""quotient, which should be 1/100 of has_percentage""")
    grouping_key: Optional[str] = Field(default=None, description="""A concatenation of fields used to group associations with the same essential/defining properties""")
    provided_by: Optional[str] = Field(default=None)
    provided_by_link: Optional[ExpandedCurie] = Field(default=None, description="""A link to the docs for the knowledge source that provided the node/edge.""")
    publications: Optional[list[str]] = Field(default=None)
    publications_links: Optional[list[ExpandedCurie]] = Field(default=None, description="""List of ExpandedCuries with id and url for publications""")
    frequency_qualifier: Optional[str] = Field(default=None)
    onset_qualifier: Optional[str] = Field(default=None)
    sex_qualifier: Optional[str] = Field(default=None)
    stage_qualifier: Optional[str] = Field(default=None)
    qualifiers: Optional[list[str]] = Field(default=None)
    qualifiers_label: Optional[str] = Field(default=None, description="""The name of the frequency_qualifier entity""")
    qualifiers_namespace: Optional[str] = Field(default=None, description="""The namespace/prefix of the frequency_qualifier entity""")
    qualifiers_category: Optional[str] = Field(default=None, description="""The category of the frequency_qualifier entity""")
    qualifier: Optional[list[str]] = Field(default=None)
    qualifier_label: Optional[str] = Field(default=None, description="""The name of the frequency_qualifier entity""")
    qualifier_namespace: Optional[str] = Field(default=None, description="""The namespace/prefix of the frequency_qualifier entity""")
    qualifier_category: Optional[str] = Field(default=None, description="""The category of the frequency_qualifier entity""")
    frequency_qualifier_label: Optional[str] = Field(default=None, description="""The name of the frequency_qualifier entity""")
    frequency_qualifier_namespace: Optional[str] = Field(default=None, description="""The namespace/prefix of the frequency_qualifier entity""")
    frequency_qualifier_category: Optional[str] = Field(default=None, description="""The category of the frequency_qualifier entity""")
    onset_qualifier_label: Optional[str] = Field(default=None, description="""The name of the onset_qualifier entity""")
    onset_qualifier_namespace: Optional[str] = Field(default=None, description="""The namespace/prefix of the onset_qualifier entity""")
    onset_qualifier_category: Optional[str] = Field(default=None, description="""The category of the onset_qualifier entity""")
    sex_qualifier_label: Optional[str] = Field(default=None, description="""The name of the sex_qualifier entity""")
    sex_qualifier_namespace: Optional[str] = Field(default=None, description="""The namespace/prefix of the sex_qualifier entity""")
    sex_qualifier_category: Optional[str] = Field(default=None, description="""The category of the sex_qualifier entity""")
    stage_qualifier_label: Optional[str] = Field(default=None, description="""The name of the stage_qualifier entity""")
    stage_qualifier_namespace: Optional[str] = Field(default=None, description="""The namespace/prefix of the stage_qualifier entity""")
    stage_qualifier_category: Optional[str] = Field(default=None, description="""The category of the stage_qualifier entity""")
    disease_context_qualifier: Optional[str] = Field(default=None, description="""A context qualifier representing a disease or condition in which a relationship expressed in an association took place.""")
    disease_context_qualifier_label: Optional[str] = Field(default=None, description="""The name of the disease_context_qualifier entity""")
    disease_context_qualifier_namespace: Optional[str] = Field(default=None, description="""The namespace/prefix of the disease_context_qualifier entity""")
    disease_context_qualifier_category: Optional[str] = Field(default=None, description="""The category of the disease_context_qualifier entity""")
    disease_context_qualifier_closure: Optional[list[str]] = Field(default=None, description="""Field containing disease_context_qualifier id and the ids of all of it's ancestors""")
    disease_context_qualifier_closure_label: Optional[list[str]] = Field(default=None, description="""Field containing disease_context_qualifier name and the names of all of it's ancestors""")
    species_context_qualifier: Optional[str] = Field(default=None, description="""A context qualifier representing a species in which a relationship expressed in an association took place.""")
    species_context_qualifier_label: Optional[str] = Field(default=None, description="""The name of the species_context_qualifier entity""")
    species_context_qualifier_namespace: Optional[str] = Field(default=None, description="""The namespace/prefix of the species_context_qualifier entity""")
    species_context_qualifier_category: Optional[str] = Field(default=None, description="""The category of the species_context_qualifier entity""")
    subject_specialization_qualifier: Optional[str] = Field(default=None, description="""A qualifier that composes with a core subject/object concept to define a more specific version of the subject concept, specifically using an ontology term that is not a subclass or descendant of the core concept and in the vast majority of cases, is of a different ontological namespace than the category or namespace of the subject identifier.""")
    subject_specialization_qualifier_label: Optional[str] = Field(default=None, description="""A label for the subject_specialization_qualifier""")
    subject_specialization_qualifier_namespace: Optional[str] = Field(default=None, description="""The namespace/prefix of the subject_specialization_qualifier""")
    subject_specialization_qualifier_category: Optional[str] = Field(default=None, description="""The category of the subject_specialization_qualifier""")
    subject_specialization_qualifier_closure: Optional[str] = Field(default=None, description="""A closure of the subject_specialization_qualifier, including the subject_specialization_qualifier itself and all of its ancestors""")
    subject_specialization_qualifier_closure_label: Optional[str] = Field(default=None, description="""A closure of the subject_specialization_qualifier, including the subject_specialization_qualifier itself and all of its ancestors""")
    object_specialization_qualifier: Optional[str] = Field(default=None, description="""A qualifier that composes with a core subject/object concept to define a more specific version of the object concept, specifically using an ontology term that is not a subclass or descendant of the core concept and in the vast majority of cases, is of a different ontological namespace than the category or namespace of the object identifier.""")
    object_specialization_qualifier_label: Optional[str] = Field(default=None, description="""A label for the object_specialization_qualifier""")
    object_specialization_qualifier_namespace: Optional[str] = Field(default=None, description="""The namespace/prefix of the object_specialization_qualifier""")
    object_specialization_qualifier_category: Optional[str] = Field(default=None, description="""The category of the object_specialization_qualifier""")
    object_specialization_qualifier_closure: Optional[str] = Field(default=None, description="""A closure of the object_specialization_qualifier, including the object_specialization_qualifier itself and all of its ancestors""")
    object_specialization_qualifier_closure_label: Optional[str] = Field(default=None, description="""A closure of the object_specialization_qualifier, including the object_specialization_qualifier itself and all of its ancestors""")


class AssociationCountList(ConfiguredBaseModel):
    """
    Container class for a list of association counts
    """
    items: list[AssociationCount] = Field(default=..., description="""A collection of items, with the type to be overriden by slot_usage""")


class CompactAssociation(ConfiguredBaseModel):
    category: Optional[str] = Field(default=None)
    subject: str = Field(default=...)
    subject_label: Optional[str] = Field(default=None, description="""The name of the subject entity""")
    predicate: str = Field(default=...)
    object: str = Field(default=...)
    object_label: Optional[str] = Field(default=None, description="""The name of the object entity""")
    negated: Optional[bool] = Field(default=None)


class AssociationTypeMapping(ConfiguredBaseModel):
    """
    A data class to hold the necessary information to produce association type counts for given  entities with appropriate directional labels
    """
    subject_label: Optional[str] = Field(default=None, description="""A label to describe the subjects of the association type as a whole for use in the UI""")
    object_label: Optional[str] = Field(default=None, description="""A label to describe the objects of the association type as a whole for use in the UI""")
    symmetric: bool = Field(default=False, description="""Whether the association type is symmetric, meaning that the subject and object labels should be interchangeable""")
    category: str = Field(default=..., description="""The biolink category to use in queries for this association type""")


class DirectionalAssociation(Association):
    """
    An association that gives it's direction relative to a specified entity
    """
    direction: AssociationDirectionEnum = Field(default=..., description="""The directionality of the association relative to a given entity for an association_count. If the entity is the subject or in the subject closure, the direction is forwards, if it is the object or in the object closure, the direction is backwards.""")
    highlighting: Optional[AssociationHighlighting] = Field(default=None, description="""Optional highlighting information for search results""")
    id: str = Field(default=...)
    category: Optional[str] = Field(default=None)
    subject: str = Field(default=...)
    original_subject: Optional[str] = Field(default=None)
    subject_namespace: Optional[str] = Field(default=None, description="""The namespace/prefix of the subject entity""")
    subject_category: Optional[str] = Field(default=None, description="""The category of the subject entity""")
    subject_closure: Optional[list[str]] = Field(default=None, description="""Field containing subject id and the ids of all of it's ancestors""")
    subject_label: Optional[str] = Field(default=None, description="""The name of the subject entity""")
    subject_closure_label: Optional[list[str]] = Field(default=None, description="""Field containing subject name and the names of all of it's ancestors""")
    subject_taxon: Optional[str] = Field(default=None)
    subject_taxon_label: Optional[str] = Field(default=None)
    predicate: str = Field(default=...)
    original_predicate: Optional[str] = Field(default=None, description="""used to hold the original relation/predicate that an external knowledge source uses before transformation to match the biolink-model specification.""")
    object: str = Field(default=...)
    original_object: Optional[str] = Field(default=None)
    object_namespace: Optional[str] = Field(default=None, description="""The namespace/prefix of the object entity""")
    object_category: Optional[str] = Field(default=None, description="""The category of the object entity""")
    object_closure: Optional[list[str]] = Field(default=None, description="""Field containing object id and the ids of all of it's ancestors""")
    object_label: Optional[str] = Field(default=None, description="""The name of the object entity""")
    object_closure_label: Optional[list[str]] = Field(default=None, description="""Field containing object name and the names of all of it's ancestors""")
    object_taxon: Optional[str] = Field(default=None)
    object_taxon_label: Optional[str] = Field(default=None)
    primary_knowledge_source: Optional[str] = Field(default=None)
    aggregator_knowledge_source: Optional[list[str]] = Field(default=None)
    negated: Optional[bool] = Field(default=None)
    pathway: Optional[str] = Field(default=None)
    evidence_count: Optional[int] = Field(default=None, description="""count of supporting documents, evidence codes, and sources supplying evidence""")
    knowledge_level: str = Field(default=..., description="""Describes the level of knowledge expressed in a statement, based on the reasoning or analysis methods used to generate the statement, or the scope or specificity of what the statement expresses to be true.""")
    agent_type: str = Field(default=..., description="""Describes the high-level category of agent who originally generated a  statement of knowledge or other type of information.""")
    has_evidence: Optional[list[str]] = Field(default=None)
    has_evidence_links: Optional[list[ExpandedCurie]] = Field(default=None, description="""List of ExpandedCuries with id and url for evidence""")
    has_count: Optional[int] = Field(default=None, description="""count of out of has_total representing a frequency""")
    has_total: Optional[int] = Field(default=None, description="""total, devided by has_count, representing a frequency""")
    has_percentage: Optional[float] = Field(default=None, description="""percentage, which may be calculated from has_count and has_total, as 100 * quotient or provided directly, rounded to the integer level""")
    has_quotient: Optional[float] = Field(default=None, description="""quotient, which should be 1/100 of has_percentage""")
    grouping_key: Optional[str] = Field(default=None, description="""A concatenation of fields used to group associations with the same essential/defining properties""")
    provided_by: Optional[str] = Field(default=None)
    provided_by_link: Optional[ExpandedCurie] = Field(default=None, description="""A link to the docs for the knowledge source that provided the node/edge.""")
    publications: Optional[list[str]] = Field(default=None)
    publications_links: Optional[list[ExpandedCurie]] = Field(default=None, description="""List of ExpandedCuries with id and url for publications""")
    frequency_qualifier: Optional[str] = Field(default=None)
    onset_qualifier: Optional[str] = Field(default=None)
    sex_qualifier: Optional[str] = Field(default=None)
    stage_qualifier: Optional[str] = Field(default=None)
    qualifiers: Optional[list[str]] = Field(default=None)
    qualifiers_label: Optional[str] = Field(default=None, description="""The name of the frequency_qualifier entity""")
    qualifiers_namespace: Optional[str] = Field(default=None, description="""The namespace/prefix of the frequency_qualifier entity""")
    qualifiers_category: Optional[str] = Field(default=None, description="""The category of the frequency_qualifier entity""")
    qualifier: Optional[list[str]] = Field(default=None)
    qualifier_label: Optional[str] = Field(default=None, description="""The name of the frequency_qualifier entity""")
    qualifier_namespace: Optional[str] = Field(default=None, description="""The namespace/prefix of the frequency_qualifier entity""")
    qualifier_category: Optional[str] = Field(default=None, description="""The category of the frequency_qualifier entity""")
    frequency_qualifier_label: Optional[str] = Field(default=None, description="""The name of the frequency_qualifier entity""")
    frequency_qualifier_namespace: Optional[str] = Field(default=None, description="""The namespace/prefix of the frequency_qualifier entity""")
    frequency_qualifier_category: Optional[str] = Field(default=None, description="""The category of the frequency_qualifier entity""")
    onset_qualifier_label: Optional[str] = Field(default=None, description="""The name of the onset_qualifier entity""")
    onset_qualifier_namespace: Optional[str] = Field(default=None, description="""The namespace/prefix of the onset_qualifier entity""")
    onset_qualifier_category: Optional[str] = Field(default=None, description="""The category of the onset_qualifier entity""")
    sex_qualifier_label: Optional[str] = Field(default=None, description="""The name of the sex_qualifier entity""")
    sex_qualifier_namespace: Optional[str] = Field(default=None, description="""The namespace/prefix of the sex_qualifier entity""")
    sex_qualifier_category: Optional[str] = Field(default=None, description="""The category of the sex_qualifier entity""")
    stage_qualifier_label: Optional[str] = Field(default=None, description="""The name of the stage_qualifier entity""")
    stage_qualifier_namespace: Optional[str] = Field(default=None, description="""The namespace/prefix of the stage_qualifier entity""")
    stage_qualifier_category: Optional[str] = Field(default=None, description="""The category of the stage_qualifier entity""")
    disease_context_qualifier: Optional[str] = Field(default=None, description="""A context qualifier representing a disease or condition in which a relationship expressed in an association took place.""")
    disease_context_qualifier_label: Optional[str] = Field(default=None, description="""The name of the disease_context_qualifier entity""")
    disease_context_qualifier_namespace: Optional[str] = Field(default=None, description="""The namespace/prefix of the disease_context_qualifier entity""")
    disease_context_qualifier_category: Optional[str] = Field(default=None, description="""The category of the disease_context_qualifier entity""")
    disease_context_qualifier_closure: Optional[list[str]] = Field(default=None, description="""Field containing disease_context_qualifier id and the ids of all of it's ancestors""")
    disease_context_qualifier_closure_label: Optional[list[str]] = Field(default=None, description="""Field containing disease_context_qualifier name and the names of all of it's ancestors""")
    species_context_qualifier: Optional[str] = Field(default=None, description="""A context qualifier representing a species in which a relationship expressed in an association took place.""")
    species_context_qualifier_label: Optional[str] = Field(default=None, description="""The name of the species_context_qualifier entity""")
    species_context_qualifier_namespace: Optional[str] = Field(default=None, description="""The namespace/prefix of the species_context_qualifier entity""")
    species_context_qualifier_category: Optional[str] = Field(default=None, description="""The category of the species_context_qualifier entity""")
    subject_specialization_qualifier: Optional[str] = Field(default=None, description="""A qualifier that composes with a core subject/object concept to define a more specific version of the subject concept, specifically using an ontology term that is not a subclass or descendant of the core concept and in the vast majority of cases, is of a different ontological namespace than the category or namespace of the subject identifier.""")
    subject_specialization_qualifier_label: Optional[str] = Field(default=None, description="""A label for the subject_specialization_qualifier""")
    subject_specialization_qualifier_namespace: Optional[str] = Field(default=None, description="""The namespace/prefix of the subject_specialization_qualifier""")
    subject_specialization_qualifier_category: Optional[str] = Field(default=None, description="""The category of the subject_specialization_qualifier""")
    subject_specialization_qualifier_closure: Optional[str] = Field(default=None, description="""A closure of the subject_specialization_qualifier, including the subject_specialization_qualifier itself and all of its ancestors""")
    subject_specialization_qualifier_closure_label: Optional[str] = Field(default=None, description="""A closure of the subject_specialization_qualifier, including the subject_specialization_qualifier itself and all of its ancestors""")
    object_specialization_qualifier: Optional[str] = Field(default=None, description="""A qualifier that composes with a core subject/object concept to define a more specific version of the object concept, specifically using an ontology term that is not a subclass or descendant of the core concept and in the vast majority of cases, is of a different ontological namespace than the category or namespace of the object identifier.""")
    object_specialization_qualifier_label: Optional[str] = Field(default=None, description="""A label for the object_specialization_qualifier""")
    object_specialization_qualifier_namespace: Optional[str] = Field(default=None, description="""The namespace/prefix of the object_specialization_qualifier""")
    object_specialization_qualifier_category: Optional[str] = Field(default=None, description="""The category of the object_specialization_qualifier""")
    object_specialization_qualifier_closure: Optional[str] = Field(default=None, description="""A closure of the object_specialization_qualifier, including the object_specialization_qualifier itself and all of its ancestors""")
    object_specialization_qualifier_closure_label: Optional[str] = Field(default=None, description="""A closure of the object_specialization_qualifier, including the object_specialization_qualifier itself and all of its ancestors""")


class ExpandedCurie(ConfiguredBaseModel):
    """
    A curie bundled along with its expanded url
    """
    id: str = Field(default=...)
    url: Optional[str] = Field(default=None)


class Entity(ConfiguredBaseModel):
    """
    Represents an Entity in the Monarch KG data model
    """
    id: str = Field(default=...)
    category: Optional[str] = Field(default=None)
    name: Optional[str] = Field(default=None)
    full_name: Optional[str] = Field(default=None, description="""The long form name of an entity""")
    deprecated: Optional[bool] = Field(default=None, description="""A boolean flag indicating that an entity is no longer considered current or valid.""")
    description: Optional[str] = Field(default=None)
    xref: Optional[list[str]] = Field(default=None)
    provided_by: Optional[str] = Field(default=None)
    in_taxon: Optional[str] = Field(default=None, description="""The biolink taxon that the entity is in the closure of.""")
    in_taxon_label: Optional[str] = Field(default=None, description="""The label of the biolink taxon that the entity is in the closure of.""")
    symbol: Optional[str] = Field(default=None)
    synonym: Optional[list[str]] = Field(default=None)
    broad_synonym: Optional[list[str]] = Field(default=None, description="""A broader synonym for the entity""")
    exact_synonym: Optional[list[str]] = Field(default=None, description="""An exact synonym for the entity""")
    narrow_synonym: Optional[list[str]] = Field(default=None, description="""A narrower synonym for the entity""")
    related_synonym: Optional[list[str]] = Field(default=None, description="""A related synonym for the entity""")
    subsets: Optional[list[str]] = Field(default=None, description="""A list of subsets that the entity belongs to""")
    uri: Optional[str] = Field(default=None, description="""The URI of the entity""")
    iri: Optional[str] = Field(default=None)
    namespace: Optional[str] = Field(default=None, description="""The namespace/prefix portion of this entity's identifier""")
    has_phenotype: Optional[list[str]] = Field(default=None, description="""A list of phenotype identifiers that are known to be associated with this entity""")
    has_phenotype_label: Optional[list[str]] = Field(default=None, description="""A list of phenotype labels that are known to be associated with this entity""")
    has_phenotype_closure: Optional[list[str]] = Field(default=None, description="""A list of phenotype identifiers that are known to be associated with this entity expanded to include all ancestors""")
    has_phenotype_closure_label: Optional[list[str]] = Field(default=None, description="""A list of phenotype labels that are known to be associated with this entity expanded to include all ancestors""")
    has_phenotype_count: Optional[int] = Field(default=None, description="""A count of the number of phenotypes that are known to be associated with this entity""")
    has_descendant: Optional[list[str]] = Field(default=None, description="""A list of entity identifiers that are known to be descendants of this entity""")
    has_descendant_label: Optional[list[str]] = Field(default=None, description="""A list of entity labels that are known to be descendants of this entity""")
    has_descendant_count: Optional[int] = Field(default=None, description="""A count of the number of entities that are known to be descendants of this entity""")


class FacetValue(ConfiguredBaseModel):
    label: str = Field(default=...)
    count: Optional[int] = Field(default=None, description="""count of documents""")


class AssociationCount(FacetValue):
    category: Optional[str] = Field(default=None)
    label: str = Field(default=...)
    count: Optional[int] = Field(default=None, description="""count of documents""")


class FacetField(ConfiguredBaseModel):
    label: str = Field(default=...)
    facet_values: Optional[list[FacetValue]] = Field(default=None, description="""Collection of FacetValue label/value instances belonging to a FacetField""")


class HistoPheno(ConfiguredBaseModel):
    id: str = Field(default=...)
    items: list[HistoBin] = Field(default=..., description="""A collection of items, with the type to be overriden by slot_usage""")


class HistoBin(FacetValue):
    id: str = Field(default=...)
    label: str = Field(default=...)
    count: Optional[int] = Field(default=None, description="""count of documents""")


class Mapping(ConfiguredBaseModel):
    """
    A minimal class to hold a SSSOM mapping
    """
    subject_id: str = Field(default=...)
    subject_label: Optional[str] = Field(default=None, description="""The name of the subject entity""")
    predicate_id: str = Field(default=...)
    object_id: str = Field(default=...)
    object_label: Optional[str] = Field(default=None, description="""The name of the object entity""")
    mapping_justification: Optional[str] = Field(default=None)
    id: str = Field(default=...)


class Node(Entity):
    """
    UI container class extending Entity with additional information
    """
    in_taxon: Optional[str] = Field(default=None, description="""The biolink taxon that the entity is in the closure of.""")
    in_taxon_label: Optional[str] = Field(default=None, description="""The label of the biolink taxon that the entity is in the closure of.""")
    inheritance: Optional[Entity] = Field(default=None)
    causal_gene: Optional[list[Entity]] = Field(default=None, description="""A list of genes that are known to be causally associated with a disease""")
    causes_disease: Optional[list[Entity]] = Field(default=None, description="""A list of diseases that are known to be causally associated with a gene""")
    mappings: Optional[list[ExpandedCurie]] = Field(default=None, description="""List of ExpandedCuries with id and url for mapped entities""")
    external_links: Optional[list[ExpandedCurie]] = Field(default=None, description="""ExpandedCurie with id and url for xrefs""")
    provided_by_link: Optional[ExpandedCurie] = Field(default=None, description="""A link to the docs for the knowledge source that provided the node/edge.""")
    association_counts: list[AssociationCount] = Field(default=...)
    node_hierarchy: Optional[NodeHierarchy] = Field(default=None)
    id: str = Field(default=...)
    category: Optional[str] = Field(default=None)
    name: Optional[str] = Field(default=None)
    full_name: Optional[str] = Field(default=None, description="""The long form name of an entity""")
    deprecated: Optional[bool] = Field(default=None, description="""A boolean flag indicating that an entity is no longer considered current or valid.""")
    description: Optional[str] = Field(default=None)
    xref: Optional[list[str]] = Field(default=None)
    provided_by: Optional[str] = Field(default=None)
    symbol: Optional[str] = Field(default=None)
    synonym: Optional[list[str]] = Field(default=None)
    broad_synonym: Optional[list[str]] = Field(default=None, description="""A broader synonym for the entity""")
    exact_synonym: Optional[list[str]] = Field(default=None, description="""An exact synonym for the entity""")
    narrow_synonym: Optional[list[str]] = Field(default=None, description="""A narrower synonym for the entity""")
    related_synonym: Optional[list[str]] = Field(default=None, description="""A related synonym for the entity""")
    subsets: Optional[list[str]] = Field(default=None, description="""A list of subsets that the entity belongs to""")
    uri: Optional[str] = Field(default=None, description="""The URI of the entity""")
    iri: Optional[str] = Field(default=None)
    namespace: Optional[str] = Field(default=None, description="""The namespace/prefix portion of this entity's identifier""")
    has_phenotype: Optional[list[str]] = Field(default=None, description="""A list of phenotype identifiers that are known to be associated with this entity""")
    has_phenotype_label: Optional[list[str]] = Field(default=None, description="""A list of phenotype labels that are known to be associated with this entity""")
    has_phenotype_closure: Optional[list[str]] = Field(default=None, description="""A list of phenotype identifiers that are known to be associated with this entity expanded to include all ancestors""")
    has_phenotype_closure_label: Optional[list[str]] = Field(default=None, description="""A list of phenotype labels that are known to be associated with this entity expanded to include all ancestors""")
    has_phenotype_count: Optional[int] = Field(default=None, description="""A count of the number of phenotypes that are known to be associated with this entity""")
    has_descendant: Optional[list[str]] = Field(default=None, description="""A list of entity identifiers that are known to be descendants of this entity""")
    has_descendant_label: Optional[list[str]] = Field(default=None, description="""A list of entity labels that are known to be descendants of this entity""")
    has_descendant_count: Optional[int] = Field(default=None, description="""A count of the number of entities that are known to be descendants of this entity""")


class NodeHierarchy(ConfiguredBaseModel):
    super_classes: list[Entity] = Field(default=...)
    sub_classes: list[Entity] = Field(default=...)


class Release(ConfiguredBaseModel):
    """
    A class to hold information about a release of the Monarch KG
    """
    version: Optional[str] = Field(default=None)
    url: Optional[str] = Field(default=None)
    kg: Optional[str] = Field(default=None)
    sqlite: Optional[str] = Field(default=None)
    solr: Optional[str] = Field(default=None)
    neo4j: Optional[str] = Field(default=None)
    metadata: Optional[str] = Field(default=None)
    graph_stats: Optional[str] = Field(default=None)
    qc_report: Optional[str] = Field(default=None)


class Results(ConfiguredBaseModel):
    limit: int = Field(default=..., description="""number of items to return in a response""")
    offset: int = Field(default=..., description="""offset into the total number of items""")
    total: int = Field(default=..., description="""total number of items matching a query""")


class AssociationResults(Results):
    items: list[Association] = Field(default=..., description="""A collection of items, with the type to be overriden by slot_usage""")
    facet_fields: Optional[list[FacetField]] = Field(default=None, description="""Collection of facet field responses with the field values and counts""")
    facet_queries: Optional[list[FacetValue]] = Field(default=None, description="""Collection of facet query responses with the query string values and counts""")
    limit: int = Field(default=..., description="""number of items to return in a response""")
    offset: int = Field(default=..., description="""offset into the total number of items""")
    total: int = Field(default=..., description="""total number of items matching a query""")


class CompactAssociationResults(Results):
    items: list[CompactAssociation] = Field(default=..., description="""A collection of items, with the type to be overriden by slot_usage""")
    facet_fields: Optional[list[FacetField]] = Field(default=None, description="""Collection of facet field responses with the field values and counts""")
    facet_queries: Optional[list[FacetValue]] = Field(default=None, description="""Collection of facet query responses with the query string values and counts""")
    limit: int = Field(default=..., description="""number of items to return in a response""")
    offset: int = Field(default=..., description="""offset into the total number of items""")
    total: int = Field(default=..., description="""total number of items matching a query""")


class AssociationTableResults(Results):
    items: list[DirectionalAssociation] = Field(default=..., description="""A collection of items, with the type to be overriden by slot_usage""")
    facet_fields: Optional[list[FacetField]] = Field(default=None, description="""Collection of facet field responses with the field values and counts""")
    facet_queries: Optional[list[FacetValue]] = Field(default=None, description="""Collection of facet query responses with the query string values and counts""")
    limit: int = Field(default=..., description="""number of items to return in a response""")
    offset: int = Field(default=..., description="""offset into the total number of items""")
    total: int = Field(default=..., description="""total number of items matching a query""")


class CategoryGroupedAssociationResults(Results):
    counterpart_category: Optional[str] = Field(default=None, description="""The category of the counterpart entity in a given association,  eg. the category of the entity that is not the subject""")
    items: list[Association] = Field(default=..., description="""A collection of items, with the type to be overriden by slot_usage""")
    limit: int = Field(default=..., description="""number of items to return in a response""")
    offset: int = Field(default=..., description="""offset into the total number of items""")
    total: int = Field(default=..., description="""total number of items matching a query""")


class EntityResults(Results):
    items: list[Entity] = Field(default=..., description="""A collection of items, with the type to be overriden by slot_usage""")
    limit: int = Field(default=..., description="""number of items to return in a response""")
    offset: int = Field(default=..., description="""offset into the total number of items""")
    total: int = Field(default=..., description="""total number of items matching a query""")


class MappingResults(Results):
    """
    SSSOM Mappings returned as a results collection
    """
    items: list[Mapping] = Field(default=..., description="""A collection of items, with the type to be overriden by slot_usage""")
    limit: int = Field(default=..., description="""number of items to return in a response""")
    offset: int = Field(default=..., description="""offset into the total number of items""")
    total: int = Field(default=..., description="""total number of items matching a query""")


class MultiEntityAssociationResults(Results):
    id: str = Field(default=...)
    name: Optional[str] = Field(default=None)
    associated_categories: list[CategoryGroupedAssociationResults] = Field(default=...)
    limit: int = Field(default=..., description="""number of items to return in a response""")
    offset: int = Field(default=..., description="""offset into the total number of items""")
    total: int = Field(default=..., description="""total number of items matching a query""")


class SearchResult(Entity):
    score: Optional[float] = Field(default=None)
    id: str = Field(default=...)
    category: str = Field(default=...)
    name: str = Field(default=...)
    full_name: Optional[str] = Field(default=None, description="""The long form name of an entity""")
    deprecated: Optional[bool] = Field(default=None, description="""A boolean flag indicating that an entity is no longer considered current or valid.""")
    description: Optional[str] = Field(default=None)
    xref: Optional[list[str]] = Field(default=None)
    provided_by: Optional[str] = Field(default=None)
    in_taxon: Optional[str] = Field(default=None, description="""The biolink taxon that the entity is in the closure of.""")
    in_taxon_label: Optional[str] = Field(default=None, description="""The label of the biolink taxon that the entity is in the closure of.""")
    symbol: Optional[str] = Field(default=None)
    synonym: Optional[list[str]] = Field(default=None)
    broad_synonym: Optional[list[str]] = Field(default=None, description="""A broader synonym for the entity""")
    exact_synonym: Optional[list[str]] = Field(default=None, description="""An exact synonym for the entity""")
    narrow_synonym: Optional[list[str]] = Field(default=None, description="""A narrower synonym for the entity""")
    related_synonym: Optional[list[str]] = Field(default=None, description="""A related synonym for the entity""")
    subsets: Optional[list[str]] = Field(default=None, description="""A list of subsets that the entity belongs to""")
    uri: Optional[str] = Field(default=None, description="""The URI of the entity""")
    iri: Optional[str] = Field(default=None)
    namespace: Optional[str] = Field(default=None, description="""The namespace/prefix portion of this entity's identifier""")
    has_phenotype: Optional[list[str]] = Field(default=None, description="""A list of phenotype identifiers that are known to be associated with this entity""")
    has_phenotype_label: Optional[list[str]] = Field(default=None, description="""A list of phenotype labels that are known to be associated with this entity""")
    has_phenotype_closure: Optional[list[str]] = Field(default=None, description="""A list of phenotype identifiers that are known to be associated with this entity expanded to include all ancestors""")
    has_phenotype_closure_label: Optional[list[str]] = Field(default=None, description="""A list of phenotype labels that are known to be associated with this entity expanded to include all ancestors""")
    has_phenotype_count: Optional[int] = Field(default=None, description="""A count of the number of phenotypes that are known to be associated with this entity""")
    has_descendant: Optional[list[str]] = Field(default=None, description="""A list of entity identifiers that are known to be descendants of this entity""")
    has_descendant_label: Optional[list[str]] = Field(default=None, description="""A list of entity labels that are known to be descendants of this entity""")
    has_descendant_count: Optional[int] = Field(default=None, description="""A count of the number of entities that are known to be descendants of this entity""")


class SearchResults(Results):
    items: list[SearchResult] = Field(default=..., description="""A collection of items, with the type to be overriden by slot_usage""")
    facet_fields: Optional[list[FacetField]] = Field(default=None, description="""Collection of facet field responses with the field values and counts""")
    facet_queries: Optional[list[FacetValue]] = Field(default=None, description="""Collection of facet query responses with the query string values and counts""")
    limit: int = Field(default=..., description="""number of items to return in a response""")
    offset: int = Field(default=..., description="""offset into the total number of items""")
    total: int = Field(default=..., description="""total number of items matching a query""")


class TextAnnotationResult(ConfiguredBaseModel):
    text: Optional[str] = Field(default=None, description="""text without tokens""")
    tokens: Optional[list[Entity]] = Field(default=None, description="""A collection of entities or concepts""")
    start: Optional[int] = Field(default=None, description="""start position of the annotation""")
    end: Optional[int] = Field(default=None, description="""end position of the annotation""")


class AssociationHighlighting(ConfiguredBaseModel):
    """
    Optional highlighting information for search results
    """
    object_label: Optional[list[str]] = Field(default=None, description="""The name of the object entity""")
    object_closure_label: Optional[list[str]] = Field(default=None, description="""Field containing object name and the names of all of it's ancestors""")
    subject_label: Optional[list[str]] = Field(default=None, description="""The name of the subject entity""")
    subject_closure_label: Optional[list[str]] = Field(default=None, description="""Field containing subject name and the names of all of it's ancestors""")
    predicate: Optional[list[str]] = Field(default=None)


# Model rebuild
# see https://pydantic-docs.helpmanual.io/usage/models/#rebuilding-a-model
PairwiseSimilarity.model_rebuild()
TermPairwiseSimilarity.model_rebuild()
TermSetPairwiseSimilarity.model_rebuild()
TermInfo.model_rebuild()
BestMatch.model_rebuild()
SemsimSearchResult.model_rebuild()
Association.model_rebuild()
AssociationCountList.model_rebuild()
CompactAssociation.model_rebuild()
AssociationTypeMapping.model_rebuild()
DirectionalAssociation.model_rebuild()
ExpandedCurie.model_rebuild()
Entity.model_rebuild()
FacetValue.model_rebuild()
AssociationCount.model_rebuild()
FacetField.model_rebuild()
HistoPheno.model_rebuild()
HistoBin.model_rebuild()
Mapping.model_rebuild()
Node.model_rebuild()
NodeHierarchy.model_rebuild()
Release.model_rebuild()
Results.model_rebuild()
AssociationResults.model_rebuild()
CompactAssociationResults.model_rebuild()
AssociationTableResults.model_rebuild()
CategoryGroupedAssociationResults.model_rebuild()
EntityResults.model_rebuild()
MappingResults.model_rebuild()
MultiEntityAssociationResults.model_rebuild()
SearchResult.model_rebuild()
SearchResults.model_rebuild()
TextAnnotationResult.model_rebuild()
AssociationHighlighting.model_rebuild()




from monarch_py.service.solr_service import SolrService, FIELD_TYPE_SUFFIXES


def test_consolidate_highlights():
    # Input highlights
    highlights = {
        "MONDO:123": {
            "name_t": ["<em>example</em> name", "another <em>example</em>"],
            "description_t": ["<em>example</em> description"],
            "name_ac": ["a third <em>example</em> name"],
        },
        "HGNC:456": {
            "name_t": ["<em>heart</em> gene"],
            "description_t": ["gene description about <em>heart</em>"],
            "name_ac": ["<em>heart</em> gene"],
        },
    }

    # Expected consolidated highlights
    expected = {
        "MONDO:123": {
            "name": ["<em>example</em> name", "a third <em>example</em> name", "another <em>example</em>"],
            "description": ["<em>example</em> description"],
        },
        "HGNC:456": {
            "name": ["<em>heart</em> gene"],
            "description": ["gene description about <em>heart</em>"],
        },
    }

    # Call the method
    result = SolrService._consolidate_highlights(highlights, FIELD_TYPE_SUFFIXES)

    # Assert the result matches the expected output
    assert result == expected


def test_strip_excluded_fields():
    # Input document
    doc = {
        "id": "123",
        "name_t": "example name",
        "description_t": "example description",
        "_version_": 1,
        "iri": "http://example.com",
        "frequency_computed_sortable_float": 0.5,
        "has_quotient_sortable_float": 0.3,
        "has_percentage_sortable_float": 0.7,
    }

    # Expected stripped document
    expected = {
        "id": "123",
    }

    # Call the method
    SolrService._strip_excluded_fields(doc)

    # Assert the result matches the expected output
    assert doc == expected


def test_strip_json():
    # Input document
    doc = {
        "id": "123",
        "name": "example name",
        "_version_": 1,
        "iri": "http://example.com",
    }

    # Expected stripped document
    expected = {
        "id": "123",
        "name": "example name",
    }

    # Call the method
    SolrService._strip_json(doc, "_version_", "iri")

    # Assert the result matches the expected output
    assert doc == expected


def test_strip_json_by_suffix():
    # Input document
    doc = {
        "id": "123",
        "name_t": "example name",
        "description_t": "example description",
        "name": "example name",
    }

    # Expected stripped document
    expected = {
        "id": "123",
        "name": "example name",
    }

    # Call the method
    SolrService._strip_json_by_suffix(doc, "_t")

    # Assert the result matches the expected output
    assert doc == expected




import pytest

@pytest.fixture
def mapping_query():
    return {'q': '*:*', 'rows': 20, 'start': 0, 'facet': True, 'facet_min_count': 1, 'facet_fields': [], 'facet_queries': [], 'filter_queries': ['subject_id:"MONDO\\:0020121" OR object_id:"MONDO\\:0020121"'], 'facet_mincount': 1, 'query_fields': None, 'def_type': 'edismax', 'q_op': 'AND', 'mm': '100%', 'boost': None, 'sort': None, 'hl': False}




import pytest

@pytest.fixture
def association_query_direct():
    return {'q': 'test:q', 'rows': 100, 'start': 100, 'facet': True, 'facet_min_count': 1, 'facet_fields': [], 'facet_queries': [], 'filter_queries': ['category:biolink\\:DiseaseToPhenotypicFeatureAssociation', 'predicate:biolink\\:causes', 'subject_closure:TEST\\:0000003', 'subject_category:biolink\\:Gene', 'subject_namespace:TEST', 'subject_taxon:NCBITaxon\\:1111', 'object_closure:TEST\\:0000004', 'object_category:biolink\\:Disease', 'object_namespace:TEST', 'object_taxon:NCBITaxon\\:2222', 'subject:TEST\\:0000001', 'object:TEST\\:0000002', 'subject:"TEST:0000005" OR object:"TEST:0000005" OR disease_context_qualifier:"TEST:0000005"'], 'facet_mincount': 1, 'query_fields': 'subject subject_label^2 subject_label_t subject_closure subject_closure_label subject_closure_label_t predicate predicate_t object object_label^2 object_label_t object_closure object_closure_label object_closure_label_t publications has_evidence primary_knowledge_source aggregator_knowledge_source provided_by ', 'def_type': 'edismax', 'q_op': 'AND', 'mm': '100%', 'boost': None, 'sort': None, 'hl': True}
